@@ -44,6 +44,7 @@ static void docRtfParaSaveProperties( SimpleOutputStream *	sos,
 				int				fromDefault,
 				int				saveIntbl,
 				int *				pPropertyChange,
+				int *				pTabsSaved,
 				int *				pCol,
 				const ParagraphProperties *	newPP,
 				const ParagraphProperties *	prevPP )
@@ -56,12 +57,12 @@ static void docRtfParaSaveProperties( SimpleOutputStream *	sos,
     if  ( ! fromDefault )
 	{
 	PROPmaskCLEAR( &updMask );
-	PROPmaskFILL( &updMask, PPprop_COUNT );
+	utilPropMaskFill( &updMask, PPprop_COUNT );
 	PROPmaskUNSET( &updMask, PPpropIN_TABLE );
 
 	docParaPropertyDifference( &updMask, newPP, prevPP, &updMask );
 
-	if  ( PROPmaskISEMPTY( &updMask ) )
+	if  ( utilPropMaskIsEmpty( &updMask ) )
 	    { return;	}
 	}
 
@@ -75,7 +76,7 @@ static void docRtfParaSaveProperties( SimpleOutputStream *	sos,
 	{ docRtfWriteTag( "\\intbl", pCol, sos );	}
 
     PROPmaskCLEAR( &updMask );
-    PROPmaskFILL( &updMask, PPprop_COUNT );
+    utilPropMaskFill( &updMask, PPprop_COUNT );
     PROPmaskUNSET( &updMask, PPpropIN_TABLE );
 
     docParaPropertyDifference( &updMask, newPP, &ppp, &updMask );
@@ -84,40 +85,20 @@ static void docRtfParaSaveProperties( SimpleOutputStream *	sos,
 
     docRtfSaveParagraphProperties( sos, pCol, &updMask, newPP );
 
+    if  ( PROPmaskISSET( &updMask, PPpropTAB_STOPS )	&&
+	  newPP->ppTabStopList.tslTabStopCount > 0	)
+	{ *pTabsSaved= 1;	}
+
     return;
     }
 
-int docRtfSaveRuler(	SimpleOutputStream *		sos,
-			const ParagraphProperties *	pp )
-    {
-    int				col= 0;
-    int				propChange= 1;
-    int				saveIntbl= 0;
-    int				noRestart= 0;
-
-    ParagraphProperties		refPP;
-
-    docInitParagraphProperties( &refPP );
-
-    docRtfWriteDestinationBegin( "\\ruler", &col, sos );
-    docRtfWriteTag( "\\pard", &col, sos );
-
-    docRtfParaSaveProperties( sos, noRestart, saveIntbl,
-					&propChange, &col, pp, &refPP );
-
-    docRtfWriteDestinationEnd( &col, sos );
-
-    docCleanParagraphProperties( &refPP );
-
-    return 0;
-    }
-
 static int docRtfSaveObject(	SimpleOutputStream *		sos,
 				int *				pCol,
 				const BufferItem *		bi,
 				const TextParticule *		tp )
     {
     InsertedObject *	io= bi->biParaObjects+ tp->tpObjectNumber;
+    PictureProperties *	pip= &(io->ioPictureProperties);
 
     docRtfWriteNextLine( pCol, sos );
 
@@ -125,9 +106,20 @@ static int docRtfSaveObject(	SimpleOutputStream *		sos,
 	{
 	case DOCokPICTWMETAFILE:
 	    docRtfWriteDestinationBegin( "\\pict", pCol, sos );
-	    docRtfWriteArgTag( "\\wmetafile", pCol, io->ioMapMode, sos );
+	    docRtfWriteArgTag( "\\wmetafile", pCol, pip->pipMapMode, sos );
 
-	    docRtfSavePictureTags( io, pCol, sos );
+	    if  ( pip->pipMetafileIsBitmap )
+		{
+		docRtfWriteTag( "\\picbmp", pCol, sos );
+
+		if  ( pip->pipMetafileBitmapBpp > 0 )
+		    {
+		    docRtfWriteArgTag( "\\picbpp",
+					pCol, pip->pipMetafileBitmapBpp, sos );
+		    }
+		}
+
+	    docRtfSavePictureTags( &(io->ioPictureProperties), pCol, sos );
 
 	    docRtfWriteMemoryBuffer( &(io->ioObjectData), pCol, sos );
 	    return 0;
@@ -136,7 +128,7 @@ static int docRtfSaveObject(	SimpleOutputStream *		sos,
 	    docRtfWriteDestinationBegin( "\\pict", pCol, sos );
 	    docRtfWriteTag( "\\macpict", pCol, sos );
 
-	    docRtfSavePictureTags( io, pCol, sos );
+	    docRtfSavePictureTags( &(io->ioPictureProperties), pCol, sos );
 
 	    docRtfWriteMemoryBuffer( &(io->ioObjectData), pCol, sos );
 
@@ -146,7 +138,7 @@ static int docRtfSaveObject(	SimpleOutputStream *		sos,
 	    docRtfWriteDestinationBegin( "\\pict", pCol, sos );
 	    docRtfWriteTag( "\\pngblip", pCol, sos );
 
-	    docRtfSavePictureTags( io, pCol, sos );
+	    docRtfSavePictureTags( &(io->ioPictureProperties), pCol, sos );
 
 	    docRtfWriteMemoryBuffer( &(io->ioObjectData), pCol, sos );
 	    return 0;
@@ -155,7 +147,7 @@ static int docRtfSaveObject(	SimpleOutputStream *		sos,
 	    docRtfWriteDestinationBegin( "\\pict", pCol, sos );
 	    docRtfWriteTag( "\\jpegblip", pCol, sos );
 
-	    docRtfSavePictureTags( io, pCol, sos );
+	    docRtfSavePictureTags( &(io->ioPictureProperties), pCol, sos );
 
 	    docRtfWriteMemoryBuffer( &(io->ioObjectData), pCol, sos );
 	    return 0;
@@ -164,14 +156,69 @@ static int docRtfSaveObject(	SimpleOutputStream *		sos,
 	    docRtfWriteDestinationBegin( "\\pict", pCol, sos );
 	    docRtfWriteTag( "\\emfblip", pCol, sos );
 
-	    docRtfSavePictureTags( io, pCol, sos );
+	    docRtfSavePictureTags( &(io->ioPictureProperties), pCol, sos );
 
 	    docRtfWriteMemoryBuffer( &(io->ioObjectData), pCol, sos );
 	    return 0;
 
 	case DOCokOLEOBJECT:
 	    docRtfWriteDestinationBegin( "\\object", pCol, sos );
-	    docRtfWriteTag( "\\objemb", pCol, sos );
+
+
+	    switch( io->ioRtfEmbedKind )
+		{
+		case EMBEDkindOBJEMB:
+		    docRtfWriteTag( "\\objemb", pCol, sos );
+		    break;
+		case EMBEDkindOBJLINK:
+		    docRtfWriteTag( "\\objlink", pCol, sos );
+		    break;
+		case EMBEDkindOBJAUTLINK:
+		    docRtfWriteTag( "\\objautlink", pCol, sos );
+		    break;
+		case EMBEDkindOBJSUB:
+		    docRtfWriteTag( "\\objsub", pCol, sos );
+		    break;
+		case EMBEDkindOBJPUB:
+		    docRtfWriteTag( "\\objpub", pCol, sos );
+		    break;
+		case EMBEDkindOBJICEMB:
+		    docRtfWriteTag( "\\objicemb", pCol, sos );
+		    break;
+		case EMBEDkindOBJHTML:
+		    docRtfWriteTag( "\\objhtml", pCol, sos );
+		    break;
+		case EMBEDkindOBJOCX:
+		    docRtfWriteTag( "\\objocx", pCol, sos );
+		    break;
+		default:
+		    LDEB(io->ioRtfEmbedKind);
+		    break;
+		}
+
+	    switch( io->ioRtfResultKind )
+		{
+		case RESULTkindUNKNOWN:
+		    break;
+		case RESULTkindRTF:
+		    docRtfWriteTag( "\\rsltrtf", pCol, sos );
+		    break;
+		case RESULTkindTXT:
+		    docRtfWriteTag( "\\rslttxt", pCol, sos );
+		    break;
+		case RESULTkindPICT:
+		    docRtfWriteTag( "\\rsltpict", pCol, sos );
+		    break;
+		case RESULTkindBMP:
+		    docRtfWriteTag( "\\rsltbmp", pCol, sos );
+		    break;
+		case RESULTkindHTML:
+		    docRtfWriteTag( "\\rslthtml", pCol, sos );
+		    break;
+		default:
+		    LDEB(io->ioRtfResultKind);
+		    break;
+		}
 
 	    if  ( io->ioObjectClass )
 		{
@@ -189,10 +236,16 @@ static int docRtfSaveObject(	SimpleOutputStream *		sos,
 
 	    docRtfWriteArgTag( "\\objw", pCol, io->ioTwipsWide, sos );
 	    docRtfWriteArgTag( "\\objh", pCol, io->ioTwipsHigh, sos );
-	    if  ( io->ioScaleX != 100 )
-		{ docRtfWriteArgTag( "\\objscalex", pCol, io->ioScaleX, sos ); }
-	    if  ( io->ioScaleY != 100 )
-		{ docRtfWriteArgTag( "\\objscaley", pCol, io->ioScaleY, sos ); }
+	    if  ( io->ioScaleXSet != 100 )
+		{
+		docRtfWriteArgTag( "\\objscalex", pCol,
+						    io->ioScaleXSet, sos );
+		}
+	    if  ( io->ioScaleYSet != 100 )
+		{
+		docRtfWriteArgTag( "\\objscaley", pCol,
+						    io->ioScaleYSet, sos );
+		}
 
 	    docRtfWriteDestinationBegin( "\\*\\objdata ", pCol, sos );
 	    docRtfWriteMemoryBuffer( &io->ioObjectData, pCol, sos );
@@ -203,9 +256,9 @@ static int docRtfSaveObject(	SimpleOutputStream *		sos,
 			    "\\result {\\pict", pCol, sos );
 
 		docRtfWriteArgTag( "\\wmetafile",
-					    pCol, io->ioResultMapMode, sos );
+					    pCol, pip->pipMapMode, sos );
 
-		docRtfSavePictureTags( io, pCol, sos );
+		docRtfSavePictureTags( &(io->ioPictureProperties), pCol, sos );
 
 		docRtfWriteMemoryBuffer( &io->ioResultData, pCol, sos );
 
@@ -215,11 +268,16 @@ static int docRtfSaveObject(	SimpleOutputStream *		sos,
 	    docRtfWriteDestinationEnd( pCol, sos );
 	    return 0;
 
-	case DOCokINCLUDEPICTURE:
+	case DOCokEPS_FILE:
+	    return 0;
+
+	case DOCokDRAWING_SHAPE:
+	    LSDEB(io->ioKind,docObjectKindStr(io->ioKind));
 	    return 0;
 
 	default:
-	    LDEB(io->ioKind); return -1;
+	    LSDEB(io->ioKind,docObjectKindStr(io->ioKind));
+	    return -1;
 	}
 
     /*  Not reached ..
@@ -255,12 +313,26 @@ static void docRtfReserveColumns(	SimpleOutputStream *	sos,
 /*									*/
 /************************************************************************/
 
+static int docRtfPopAttribute(		RtfWritingContext *	rwc )
+    {
+    PushedAttribute *	pa;
+
+    pa= rwc->rwcOutsideFldrsltAttribute;
+    if  ( ! pa )
+	{ XDEB(pa); return -1;	}
+
+    rwc->rwcTextAttribute= pa->paTextAttribute;
+    rwc->rwcOutsideFldrsltAttribute= pa->paNext;
+
+    free( pa );
+
+    return 0;
+    }
+
 static void docRtfFinishFldrslt(	SimpleOutputStream *	sos,
 					int *			pCol,
 					RtfWritingContext *	rwc )
     {
-    PushedAttribute *	pa;
-
     docRtfWriteDestinationEnd( pCol, sos );
     docRtfWriteDestinationEnd( pCol, sos );
     docRtfWriteDestinationEnd( pCol, sos );
@@ -268,24 +340,31 @@ static void docRtfFinishFldrslt(	SimpleOutputStream *	sos,
     rwc->rwcHasPrecedingTags= 0;
     rwc->rwcInFldrslt--;
 
-    pa= rwc->rwcOutsideFldrsltAttribute;
-    if  ( ! pa )
-	{ XDEB(pa); return;	}
-
-    rwc->rwcTextAttribute= pa->paTextAttribute;
-    rwc->rwcOutsideFldrsltAttribute= pa->paNext;
-
-    free( pa );
+    if  ( docRtfPopAttribute( rwc ) )
+	{ LDEB(1);	}
 
     return;
     }
 
+static int docRtfPushAttribute(		RtfWritingContext *	rwc )
+    {
+    PushedAttribute *		pa;
+
+    pa= malloc( sizeof( PushedAttribute ) );
+    if  ( ! pa )
+	{ XDEB(pa); return- 1;	}
+    pa->paNext= rwc->rwcOutsideFldrsltAttribute;
+    rwc->rwcOutsideFldrsltAttribute= pa;
+    pa->paTextAttribute= rwc->rwcTextAttribute;
+
+    return 0;
+    }
+
 static void docRtfWriteStartField(	SimpleOutputStream *	sos,
 					const DocumentField *	df,
 					int *			pCol,
 					RtfWritingContext *	rwc )
     {
-    PushedAttribute *		pa;
     const unsigned char *	bytes= df->dfInstructions.mbBytes;
     int				byteCount= df->dfInstructions.mbSize;
 
@@ -310,12 +389,8 @@ static void docRtfWriteStartField(	SimpleOutputStream *	sos,
     rwc->rwcHasPrecedingTags= 0;
     rwc->rwcInFldrslt++;
 
-    pa= malloc( sizeof( PushedAttribute ) );
-    if  ( ! pa )
-	{ XDEB(pa); return;	}
-    pa->paNext= rwc->rwcOutsideFldrsltAttribute;
-    rwc->rwcOutsideFldrsltAttribute= pa;
-    pa->paTextAttribute= rwc->rwcTextAttribute;
+    if  ( docRtfPushAttribute( rwc ) )
+	{ LDEB(1);	}
 
     return;
     }
@@ -366,152 +441,6 @@ static void docRtfFinishBookmark(	SimpleOutputStream *	sos,
     return;
     }
 
-/************************************************************************/
-/*									*/
-/*  Save Cell properties.						*/
-/*									*/
-/************************************************************************/
-
-static void docRtfSaveCellProperties(	const CellProperties *	cp,
-					int *			pCol,
-					int			shiftLeft,
-					SimpleOutputStream *	sos )
-    {
-    int		didAnything= 0;
-
-    if  ( cp->cpLeftInMergedRow )
-	{ docRtfWriteTag( "\\clmgf", pCol, sos ); didAnything= 1;	}
-    if  ( cp->cpMergedWithLeft )
-	{ docRtfWriteTag( "\\clmrg", pCol, sos ); didAnything= 1;	}
-    if  ( cp->cpTopInMergedColumn )
-	{ docRtfWriteTag( "\\clvmgf", pCol, sos ); didAnything= 1;	}
-    if  ( cp->cpMergedWithAbove )
-	{ docRtfWriteTag( "\\clvmrg", pCol, sos ); didAnything= 1;	}
-
-    docRtfWriteNextLine( pCol, sos );
-
-    if  ( cp->cpTopBorder.bpStyle != DOCbsNONE )
-	{ docRtfSaveBorder( "\\clbrdrt", pCol, &(cp->cpTopBorder), sos ); }
-    if  ( cp->cpLeftBorder.bpStyle != DOCbsNONE )
-	{ docRtfSaveBorder( "\\clbrdrl", pCol, &(cp->cpLeftBorder), sos ); }
-    if  ( cp->cpRightBorder.bpStyle != DOCbsNONE )
-	{ docRtfSaveBorder( "\\clbrdrr", pCol, &(cp->cpRightBorder), sos ); }
-    if  ( cp->cpBottomBorder.bpStyle != DOCbsNONE )
-	{ docRtfSaveBorder( "\\clbrdrb", pCol, &(cp->cpBottomBorder), sos ); }
-
-    switch( cp->cpShadingPattern )
-	{
-	case DOCspSOLID:
-	    break;
-
-	case DOCspBGBDIAG:
-	    docRtfWriteTag( "\\clbgbdiag", pCol, sos );
-	    break;
-	case DOCspBGCROSS:
-	    docRtfWriteTag( "\\clbgcross", pCol, sos );
-	    break;
-	case DOCspBGDCROSS:
-	    docRtfWriteTag( "\\clbgdcross", pCol, sos );
-	    break;
-	case DOCspBGDKBDIAG:
-	    docRtfWriteTag( "\\clbgdkbdiag", pCol, sos );
-	    break;
-	case DOCspBGDKCROSS:
-	    docRtfWriteTag( "\\clbgdkcross", pCol, sos );
-	    break;
-	case DOCspBGDKDCROSS:
-	    docRtfWriteTag( "\\clbgdkdcross", pCol, sos );
-	    break;
-	case DOCspBGDKFDIAG:
-	    docRtfWriteTag( "\\clbgdkfdiag", pCol, sos );
-	    break;
-	case DOCspBGDKHORIZ:
-	    docRtfWriteTag( "\\clbgdkhor", pCol, sos );
-	    break;
-	case DOCspBGDKVERT:
-	    docRtfWriteTag( "\\clbgdkvert", pCol, sos );
-	    break;
-	case DOCspBGFDIAG:
-	    docRtfWriteTag( "\\clbgfdiag", pCol, sos );
-	    break;
-	case DOCspBGHORIZ:
-	    docRtfWriteTag( "\\clbghoriz", pCol, sos );
-	    break;
-	case DOCspBGVERT:
-	    docRtfWriteTag( "\\clbgvert", pCol, sos );
-	    break;
-
-	default:
-	    LDEB(cp->cpShadingPattern);
-	    break;
-	}
-
-    docRtfWriteArgTag( "\\cellx", pCol,
-				cp->cpRightBoundaryTwips- shiftLeft, sos );
-
-    docRtfWriteNextLine( pCol, sos );
-    }
-
-static void docRtfSaveRowProperties(	const RowProperties *	rp,
-					int *			pCol,
-					int			col0,
-					int			col1,
-					SimpleOutputStream *	sos )
-    {
-    const CellProperties *	cp= rp->rpCells;
-
-    int				col;
-    int				shiftLeft= 0;
-
-    docRtfWriteTag( "\\trowd", pCol, sos );
-
-    if  ( rp->rpHalfGapWidthTwips != 0 )
-	{ docRtfWriteArgTag( "\\trgaph", pCol, rp->rpHalfGapWidthTwips, sos ); }
-
-    if  ( rp->rpLeftIndentTwips != 0 )
-	{ docRtfWriteArgTag( "\\trleft", pCol, rp->rpLeftIndentTwips, sos ); }
-
-    if  ( rp->rpHeightTwips != 0 )
-	{ docRtfWriteArgTag( "\\trrh", pCol, rp->rpHeightTwips, sos ); }
-
-    if  ( rp->rpIsTableHeader )
-	{ docRtfWriteTag( "\\trhdr", pCol, sos ); }
-    if  ( rp->rpKeepOnOnePage )
-	{ docRtfWriteTag( "\\trkeep", pCol, sos ); }
-
-    docRtfWriteNextLine( pCol, sos );
-
-    if  ( rp->rpTopBorder.bpStyle != DOCbsNONE )
-	{ docRtfSaveBorder( "\\trbrdrt", pCol, &(rp->rpTopBorder), sos ); }
-    if  ( rp->rpLeftBorder.bpStyle != DOCbsNONE )
-	{ docRtfSaveBorder( "\\trbrdrl", pCol, &(rp->rpLeftBorder), sos ); }
-    if  ( rp->rpRightBorder.bpStyle != DOCbsNONE )
-	{ docRtfSaveBorder( "\\trbrdrr", pCol, &(rp->rpRightBorder), sos ); }
-    if  ( rp->rpBottomBorder.bpStyle != DOCbsNONE )
-	{ docRtfSaveBorder( "\\trbrdrb", pCol, &(rp->rpBottomBorder), sos ); }
-
-    if  ( rp->rpHasVerticalBorders )
-	{
-	docRtfSaveBorder( "\\trbrdrv", pCol, &(rp->rpVerticalBorder), sos );
-	}
-    if  ( rp->rpHasHorizontalBorders )
-	{
-	docRtfSaveBorder( "\\trbrdrh", pCol, &(rp->rpHorizontalBorder), sos );
-	}
-
-    if  ( col0 > 0 )
-	{ shiftLeft= cp[col0-1].cpRightBoundaryTwips; }
-
-    for ( col= 0; col < rp->rpCellCount; cp++, col++ )
-	{
-	if  ( ( col0 < 0 || col >= col0 )	&&
-	      ( col1 < 0 || col <= col1 )	)
-	    { docRtfSaveCellProperties( cp, pCol, shiftLeft, sos );	}
-	}
-
-    return;
-    }
-
 /************************************************************************/
 /*									*/
 /*  Enter/Leave a table.						*/
@@ -530,6 +459,9 @@ static int docRtfPushTable(	RtfWritingContext *		rwc,
     int			col0= -1;
     int			col1= -1;
 
+    const int * const	colorMap= (const int *)0;
+    const int * const	listStyleMap= (const int *)0;
+
     docRtfWriteNextLine( pCol, sos );
     docRtfWriteDestinationBegin( "", pCol, sos );
 
@@ -540,11 +472,12 @@ static int docRtfPushTable(	RtfWritingContext *		rwc,
     PROPmaskCLEAR( &ppChgMask );
 
     PROPmaskCLEAR( &ppUpdMask );
-    PROPmaskFILL( &ppUpdMask, PPprop_COUNT );
+    utilPropMaskFill( &ppUpdMask, PPprop_COUNT );
 
     if  ( docUpdParaProperties( &ppChgMask,
 			    &(rwc->rwcOutsideTableParagraphProperties),
-			    &ppUpdMask, &(rwc->rwcParagraphProperties) ) )
+			    &ppUpdMask, &(rwc->rwcParagraphProperties),
+			    colorMap, listStyleMap ) )
 	{ LDEB(1); return -1;	}
 
     if  ( ds )
@@ -553,7 +486,7 @@ static int docRtfPushTable(	RtfWritingContext *		rwc,
     docRtfSaveRowProperties( &(rowBi->biRowProperties), pCol, col0, col1, sos );
 
     if  ( docCopyRowProperties( &(rwc->rwcRowProperties),
-					&(rowBi->biRowProperties) ) )
+				&(rowBi->biRowProperties), (const int *)0 ) )
 	{ LDEB(1); return -1;	}
 
     return 0;
@@ -566,6 +499,9 @@ static int docRtfPopTable(	RtfWritingContext *	rwc,
     PropertyMask	ppChgMask;
     PropertyMask	ppUpdMask;
 
+    const int * const	colorMap= (const int *)0;
+    const int * const	listStyleMap= (const int *)0;
+
     docRtfWriteDestinationEnd( pCol, sos );
     docRtfWriteNextLine( pCol, sos );
 
@@ -579,11 +515,12 @@ static int docRtfPopTable(	RtfWritingContext *	rwc,
     PROPmaskCLEAR( &ppChgMask );
 
     PROPmaskCLEAR( &ppUpdMask );
-    PROPmaskFILL( &ppUpdMask, PPprop_COUNT );
+    utilPropMaskFill( &ppUpdMask, PPprop_COUNT );
 
     if  ( docUpdParaProperties( &ppChgMask,
 			    &(rwc->rwcParagraphProperties), &ppUpdMask,
-			    &(rwc->rwcOutsideTableParagraphProperties) ) )
+			    &(rwc->rwcOutsideTableParagraphProperties),
+			    colorMap, listStyleMap ) )
 	{ LDEB(1); return -1;	}
 
     return 0;
@@ -619,7 +556,7 @@ int docRtfSaveExternalItem(		SimpleOutputStream *	sos,
 
     docCleanParagraphProperties( &(rwc->rwcParagraphProperties) );
     docInitParagraphProperties( &(rwc->rwcParagraphProperties) );
-    docInitTextAttribute( &(rwc->rwcTextAttribute) );
+    utilInitTextAttribute( &(rwc->rwcTextAttribute) );
 
     rwc->rwcInTable= 0;
 
@@ -636,7 +573,7 @@ int docRtfSaveExternalItem(		SimpleOutputStream *	sos,
 
     docCleanParagraphProperties( &(rwc->rwcParagraphProperties) );
     docInitParagraphProperties( &(rwc->rwcParagraphProperties) );
-    docInitTextAttribute( &(rwc->rwcTextAttribute) );
+    utilInitTextAttribute( &(rwc->rwcTextAttribute) );
 
     while( rwc->rwcInFldrslt > 0 )
 	{ docRtfFinishFldrslt( sos, pCol, rwc );	}
@@ -666,34 +603,70 @@ int docRtfSaveExternalItem(		SimpleOutputStream *	sos,
 
 static void docRtfSwitchTextAttributes(	SimpleOutputStream *	sos,
 					int *			pCol,
+					const BufferDocument *	bd,
 					const TextParticule *	tp,
 					RtfWritingContext *	rwc )
     {
-    if  ( tp->tpTextAttribute.taFontNumber >= 0 )
+    TextAttribute		ta;
+
+    utilGetTextAttributeByNumber( &ta, &(bd->bdTextAttributeList),
+						tp->tpTextAttributeNumber );
+
+    if  ( ta.taFontNumber >= 0 )
 	{
 	PropertyMask	chgMask;
 	PropertyMask	updMask;
 
 	PROPmaskCLEAR( &chgMask );
-	PROPmaskFILL( &chgMask, TAprop_COUNT );
+	utilPropMaskFill( &chgMask, TAprop_COUNT );
 
 	PROPmaskCLEAR( &updMask );
 
-	PROPmaskUNSET( &chgMask, TApropSHOWASLINK );
-
-	docUpdateTextAttribute( &updMask, &(rwc->rwcTextAttribute),
-					    tp->tpTextAttribute, &chgMask );
-	if  ( ! PROPmaskISEMPTY( &updMask ) )
+	utilUpdateTextAttribute( &updMask, &(rwc->rwcTextAttribute),
+							    &ta, &chgMask );
+	if  ( ! utilPropMaskIsEmpty( &updMask ) )
 	    {
-	    docRtfSaveTextAttribute( sos, pCol,
-				    &updMask, &(tp->tpTextAttribute) );
+	    const DocumentProperties *		dp= &(bd->bdProperties);
+	    const DocumentFontList *		dfl= &(dp->dpFontList);
+	    const DocumentFont *		df;
+	    const OfficeCharsetMapping *	ocm;
+
+	    docRtfSaveTextAttribute( sos, pCol, &updMask, &ta );
 	    rwc->rwcHasPrecedingTags= 1;
+
+	    rwc->rwcTextOutputMapping= rwc->rwcDefaultOutputMapping;
+
+	    df= dfl->dflFonts+ ta.taFontNumber;
+	    ocm= df->dfOfficeCharsetMapping;
+
+	    if  ( ! ocm )
+		{
+		/* SXDEB(df->dfName,ocm); */
+		rwc->rwcTextOutputMapping= (const unsigned char *)0;
+		}
+	    else{
+		if  ( ocm->ocmFromX11ToOffice )
+		    { rwc->rwcTextOutputMapping= ocm->ocmFromX11ToOffice; }
+		}
 	    }
 	}
 
     return;
     }
 
+/************************************************************************/
+/*									*/
+/*  Save a paragraph to rtf.						*/
+/*									*/
+/*  8)  MS-Word has a tendency to only pick up certain paragraph	*/
+/*	properties, such as the tabs, at the paragraph mark. On the	*/
+/*	other do superfluous paragraph marks result in empty paragraphs.*/
+/*	Try to find a compromise: The \cell or \row count as a paragraph*/
+/*	mark, so no need to save a paragraph mark in a table. This	*/
+/*	construct was necessary to activate tabs in headers and footers.*/
+/*									*/
+/************************************************************************/
+
 static int docRtfSaveParaItem(	SimpleOutputStream *		sos,
 				int *				pCol,
 				const BufferDocument *		bd,
@@ -711,6 +684,7 @@ static int docRtfSaveParaItem(	SimpleOutputStream *		sos,
     int					saveIntbl;
     int					restartFromDefault= 0;
     int					propChange= 0;
+    int					tabsSaved= 0;
 
     const DocumentField *		df;
     const FieldKindInformation *	fki;
@@ -718,6 +692,9 @@ static int docRtfSaveParaItem(	SimpleOutputStream *		sos,
     PropertyMask			ppChgMask;
     PropertyMask			ppUpdMask;
 
+    const int * const			colorMap= (const int *)0;
+    const int * const			listStyleMap= (const int *)0;
+
     saveIntbl= ! ds || ! docSelectionInsideCell( ds );
 
     if  ( saveIntbl )
@@ -731,7 +708,7 @@ static int docRtfSaveParaItem(	SimpleOutputStream *		sos,
 	}
 
     docRtfParaSaveProperties( sos, restartFromDefault, saveIntbl,
-					&propChange, pCol,
+					&propChange, &tabsSaved, pCol,
 					&(bi->biParaProperties),
 					&(rwc->rwcParagraphProperties) );
 
@@ -741,7 +718,7 @@ static int docRtfSaveParaItem(	SimpleOutputStream *		sos,
     if  ( restartFromDefault || propChange )
 	{
 	/*  1  */
-	docInitTextAttribute( &(rwc->rwcTextAttribute) );
+	utilInitTextAttribute( &(rwc->rwcTextAttribute) );
 	rwc->rwcTextAttribute.taFontSizeHalfPoints= 24;
 
 	/*  2  */
@@ -754,8 +731,12 @@ static int docRtfSaveParaItem(	SimpleOutputStream *		sos,
 
     if  ( ds && ds->dsBegin.dpBi == bi )
 	{
-	part= ds->dsBegin.dpParticule;
+	const int	lastOne= 1;
+
 	stroff= ds->dsBegin.dpStroff;
+
+	if  ( docFindParticuleOfPosition( &part, &(ds->dsBegin), lastOne ) )
+	    { LDEB(stroff); return -1;	}
 	}
 
     tp= bi->biParaParticules+ part;
@@ -767,7 +748,9 @@ static int docRtfSaveParaItem(	SimpleOutputStream *		sos,
 
     for ( ; part < bi->biParaParticuleCount; part++, tp++ )
 	{
-	docRtfSwitchTextAttributes( sos, pCol, tp, rwc );
+	if  ( tp->tpKind == DOCkindTEXT		||
+	      tp->tpStrlen > 0			)
+	    { docRtfSwitchTextAttributes( sos, pCol, bd, tp, rwc ); }
 
 	if  ( rwc->rwcSaveAsLink )
 	    {
@@ -802,6 +785,14 @@ static int docRtfSaveParaItem(	SimpleOutputStream *		sos,
 		rwc->rwcHasPrecedingTags= 1;
 		continue;
 
+	    case DOCkindCHFTNSEPC:
+		if  ( stroffUpto >= 0 && stroff >= stroffUpto )
+		    { break;	}
+
+		docRtfWriteTag( "\\chftnsepc", pCol, sos ); s++; stroff++;
+		rwc->rwcHasPrecedingTags= 1;
+		continue;
+
 	    case DOCkindTAB:
 		if  ( stroffUpto >= 0 && stroff >= stroffUpto )
 		    { break;	}
@@ -830,6 +821,16 @@ static int docRtfSaveParaItem(	SimpleOutputStream *		sos,
 		rwc->rwcHasPrecedingTags= 0;
 		continue;
 
+	    case DOCkindCOLUMNBREAK:
+		if  ( stroffUpto >= 0 && stroff >= stroffUpto )
+		    { break;	}
+
+		docRtfWriteTag( "\\column", pCol, sos );
+		docRtfWriteNextLine( pCol, sos );
+		s += tp->tpStrlen; stroff += tp->tpStrlen; /* += 1 */
+		rwc->rwcHasPrecedingTags= 0;
+		continue;
+
 	    case DOCkindTEXT:
 		if  ( stroffUpto >= 0 && stroff >= stroffUpto )
 		    { break;	}
@@ -851,7 +852,7 @@ static int docRtfSaveParaItem(	SimpleOutputStream *		sos,
 		    rwc->rwcHasPrecedingTags= 0;
 		    }
 
-		docRtfEscapeString( s, rwc->rwcOutputMapping, pCol, n, sos );
+		docRtfEscapeString( s, rwc->rwcTextOutputMapping, pCol, n, sos );
 		stroff += n; s += n;
 
 		continue;
@@ -927,6 +928,14 @@ static int docRtfSaveParaItem(	SimpleOutputStream *		sos,
 		    count= docCountParticulesInField( bi, part,
 						bi->biParaParticuleCount );
 
+		    if  ( count > 0 && tp[1].tpKind == DOCkindTEXT )
+			{
+			docRtfSwitchTextAttributes( sos, pCol, bd, tp+ 1, rwc );
+			}
+		    else{
+			docRtfSwitchTextAttributes( sos, pCol, bd, tp, rwc );
+			}
+
 		    docRtfWriteTag( "\\chftn", pCol, sos );
 		    rwc->rwcHasPrecedingTags= 1;
 
@@ -938,30 +947,54 @@ static int docRtfSaveParaItem(	SimpleOutputStream *		sos,
 		    continue;
 		    }
 
+		if  ( df->dfKind == DOCfkLISTTEXT	&&
+		      bi->biParaListOverride > 0	)
+		    {
+		    if  ( docRtfPushAttribute( rwc ) )
+			{ LDEB(1);	}
+
+		    docRtfWriteDestinationBegin( "\\listtext", pCol, sos );
+		    rwc->rwcHasPrecedingTags= 1;
+		    }
+
 		s += tp->tpStrlen; stroff += tp->tpStrlen; /* += 0 */
 		continue;
 
 	    case DOCkindFIELDEND:
-		df= bd->bdFieldList.dflFields+ tp->tpObjectNumber;
-		fki= DOC_FieldKinds+ df->dfKind;
+		if  ( tp->tpObjectNumber < 0 )
+		    { LDEB(tp->tpObjectNumber); }
+		else{
+		    df= bd->bdFieldList.dflFields+ tp->tpObjectNumber;
+		    fki= DOC_FieldKinds+ df->dfKind;
 
-		if  ( df->dfKind == DOCfkBOOKMARK	&&
-		      rwc->rwcSaveBookmarks		)
-		    {
-		    docRtfFinishBookmark( sos, df, pCol, rwc );
-		    rwc->rwcHasPrecedingTags= 0;
+		    if  ( df->dfKind == DOCfkBOOKMARK	&&
+			  rwc->rwcSaveBookmarks		)
+			{
+			docRtfFinishBookmark( sos, df, pCol, rwc );
+			rwc->rwcHasPrecedingTags= 0;
+			}
+
+		    if  ( fki->fkiIsFieldInRtf		&&
+			  fki->fkiLevel == DOClevTEXT	)
+			{
+			if  ( rwc->rwcInFldrslt )
+			    { docRtfFinishFldrslt( sos, pCol, rwc );	}
+			}
+
+		    if  ( df->dfKind == DOCfkCHFTN )
+			{ /* nothing */	}
+
+		    if  ( df->dfKind == DOCfkLISTTEXT	&&
+			  bi->biParaListOverride > 0	)
+			{
+			docRtfWriteDestinationEnd( pCol, sos );
+			rwc->rwcHasPrecedingTags= 0;
+
+			if  ( docRtfPopAttribute( rwc ) )
+			    { LDEB(1);	}
+			}
 		    }
 
-		if  ( fki->fkiIsFieldInRtf		&&
-		      fki->fkiLevel == DOClevTEXT	)
-		    {
-		    if  ( rwc->rwcInFldrslt )
-			{ docRtfFinishFldrslt( sos, pCol, rwc );	}
-		    }
-
-		if  ( df->dfKind == DOCfkCHFTN )
-		    { /* nothing */	}
-
 		s += tp->tpStrlen; stroff += tp->tpStrlen; /* += 0 */
 		rwc->rwcHasPrecedingTags= 0;
 		continue;
@@ -1035,8 +1068,10 @@ static int docRtfSaveParaItem(	SimpleOutputStream *		sos,
 	while( rwc->rwcInFldrslt > 0 )
 	    { docRtfFinishFldrslt( sos, pCol, rwc ); }
 
+	/*  8  */
 	if  ( ( ! bi->biParaInTable				&&
 	        bi->biInExternalItem == DOCinBODY		)	||
+	      ( tabsSaved && ! bi->biParaInTable )			||
 	      bi->biNumberInParent < bi->biParent->biChildCount- 1	)
 	    {
 	    docRtfWriteTag( "\\par", pCol, sos );
@@ -1048,10 +1083,11 @@ static int docRtfSaveParaItem(	SimpleOutputStream *		sos,
     PROPmaskCLEAR( &ppChgMask );
 
     PROPmaskCLEAR( &ppUpdMask );
-    PROPmaskFILL( &ppUpdMask, PPprop_COUNT );
+    utilPropMaskFill( &ppUpdMask, PPprop_COUNT );
 
     if  ( docUpdParaProperties( &ppChgMask, &(rwc->rwcParagraphProperties),
-				    &ppUpdMask, &(bi->biParaProperties) ) )
+			&ppUpdMask, &(bi->biParaProperties),
+			colorMap, listStyleMap ) )
 	{ LDEB(1); return -1;	}
 
     return 0;
@@ -1143,15 +1179,15 @@ static int docSaveSectionProperties(	SimpleOutputStream *		sos,
     docInitSectionProperties( &spDef );
 
     PROPmaskCLEAR( &updMask );
-    PROPmaskFILL( &updMask, DGprop_COUNT );
+    utilPropMaskFill( &updMask, DGprop_COUNT );
     PROPmaskUNSET( &updMask, DGpropHEADER_POSITION );
     PROPmaskUNSET( &updMask, DGpropFOOTER_POSITION );
 
-    appSetDocumentGeometry( &(spDef.spDocumentGeometry),
+    utilUpdDocumentGeometry( &(spDef.spDocumentGeometry),
 				    &(dp->dpGeometry), &updMask, &updMask );
 
     PROPmaskCLEAR( &updMask );
-    PROPmaskFILL( &updMask, SPprop_COUNT );
+    utilPropMaskFill( &updMask, SPprop_COUNT );
 
     docSectPropertyDifference( &updMask, &spDef, sp, &updMask );
 
@@ -1163,7 +1199,7 @@ static int docSaveSectionProperties(	SimpleOutputStream *		sos,
     docRtfWriteTag( "\\sectd", pCol, sos );
 
     docRtfSaveSectionProperties( sos,
-				rwc->rwcOutputMapping, pCol, &updMask, sp );
+			rwc->rwcDefaultOutputMapping, pCol, &updMask, sp );
 
     return 0;
     }
@@ -1276,11 +1312,11 @@ static void docRtfInitWritingContext(	RtfWritingContext *	rwc )
     {
     int		i;
 
-    docInitTextAttribute( &(rwc->rwcTextAttribute) );
+    utilInitTextAttribute( &(rwc->rwcTextAttribute) );
     docInitParagraphProperties( &(rwc->rwcParagraphProperties) );
     docInitRowProperties( &(rwc->rwcRowProperties) );
 
-    docInitTextAttribute( &(rwc->rwcOutsideTableTextAttribute) );
+    utilInitTextAttribute( &(rwc->rwcOutsideTableTextAttribute) );
     docInitParagraphProperties( &(rwc->rwcOutsideTableParagraphProperties) );
 
     rwc->rwcOutsideFldrsltAttribute= (PushedAttribute *)0;
@@ -1298,7 +1334,9 @@ static void docRtfInitWritingContext(	RtfWritingContext *	rwc )
     docInitField( &(rwc->rwcSaveAsPagerefField) );
 
     for ( i= 0; i < 256; i++ )
-	{ rwc->rwcOutputMapping[i]= i;	}
+	{ rwc->rwcDefaultOutputMapping[i]= i;	}
+
+    rwc->rwcTextOutputMapping= rwc->rwcDefaultOutputMapping;
     }
 
 static void docRtfCleanWritingContext(	RtfWritingContext *	rwc )
@@ -1336,7 +1374,7 @@ int docRtfSaveDocument(	SimpleOutputStream *		sos,
     docRtfInitWritingContext( &rwc );
     rwc.rwcSaveBookmarks= saveBookmarks;
 
-    docRtfWriteDestinationBegin( "\\rtf0\\ansi", &col, sos );
+    docRtfWriteDestinationBegin( "\\rtf1\\ansi", &col, sos );
     docRtfWriteNextLine( &col, sos );
 
     if  ( docRtfSaveDocumentProperties( sos, &col, &rwc, bd ) )
