@@ -23,6 +23,15 @@
 /*									*/
 /*  Find the first misspelling from a certain position in a paragraph.	*/
 /*									*/
+/*  1)	Skip initial particules: Find the first particule that starts	*/
+/*	after the starting point for spell checking.			*/
+/*  2)	Skip particules that can be included in a spell check range.	*/
+/*  3)	If spell checking starts in the middle of a word, skip to the	*/
+/*	end of the word.						*/
+/*  4)	Do not extend the string beyond the beginning/end of a field.	*/
+/*									*/
+/*  TODO: Do not check the contents of fields that are not editable.	*/
+/*									*/
 /************************************************************************/
 
 int docSpellParaFindNext(	DocumentSelection *		ds,
@@ -37,13 +46,40 @@ int docSpellParaFindNext(	DocumentSelection *		ds,
     int				acceptedPos= stroff;
     const unsigned char *	str= paraBi->biParaString+ stroff;
 
+    const int			lastOne= 1;
+    int				part;
+    const TextParticule *	tp;
+
+    int				stroffUpto;
+
     indInitSpellScanJob( &ssj );
 
+    if  ( docFindParticule( &part, paraBi, stroff, lastOne ) )
+	{ LDEB(stroff); return -1;	}
+
+    tp= paraBi->biParaParticules+ part;
+
+    /*  1  */
+    while( part < paraBi->biParaParticuleCount	&&
+	   tp->tpStroff <= stroff		)
+	{ part++; tp++;	}
+
+    /*  2  */
+    while( part < paraBi->biParaParticuleCount	&&
+	   ! DOCkindBETWEEN_TEXT( tp->tpKind )	)
+	{ part++; tp++;	}
+
+    /*  3  */
     if  ( stroff > 0						&&
 	  ( scc->sccCharKinds[ str[ 0] ] & CHARisALNUM )	&&
 	  ( scc->sccCharKinds[ str[-1] ] & CHARisALNUM )	)
 	{
-	while( stroff < paraBi->biParaStrlen			&&
+	/*  4  */
+	if  ( part < paraBi->biParaParticuleCount )
+	    { stroffUpto= tp->tpStroff;		}
+	else{ stroffUpto= paraBi->biParaStrlen; }
+
+	while( stroff < stroffUpto				&&
 	       ( scc->sccCharKinds[ *str ] & CHARisALNUM )	)
 	    { stroff++; str++; }
 	}
@@ -52,7 +88,22 @@ int docSpellParaFindNext(	DocumentSelection *		ds,
 	{
 	int		count;
 
-	while( stroff < paraBi->biParaStrlen			&&
+	/*  1  */
+	while( part < paraBi->biParaParticuleCount	&&
+	       tp->tpStroff <= stroff		)
+	    { part++; tp++;	}
+
+	/*  2  */
+	while( part < paraBi->biParaParticuleCount	&&
+	       ! DOCkindBETWEEN_TEXT( tp->tpKind )	)
+	    { part++; tp++;	}
+
+	/*  4  */
+	if  ( part < paraBi->biParaParticuleCount )
+	    { stroffUpto= tp->tpStroff;		}
+	else{ stroffUpto= paraBi->biParaStrlen; }
+
+	while( stroff < stroffUpto				&&
 	       ! ( scc->sccCharKinds[ *str ] & CHARisALNUM )	)
 	    {
 	    indAddCharacterToPossibilities( &ssj, *str );
@@ -61,6 +112,8 @@ int docSpellParaFindNext(	DocumentSelection *		ds,
 
 	if  ( stroff >= paraBi->biParaStrlen )
 	    { break;	}
+	if  ( stroff >= stroffUpto )
+	    { continue;	}
 
 	indAddCharacterToPossibilities( &ssj, *str );
 
@@ -68,7 +121,7 @@ int docSpellParaFindNext(	DocumentSelection *		ds,
 	    { CDEB(*str); return -1;	}
 
 	stroff++; str++;
-	while( stroff < paraBi->biParaStrlen		&&
+	while( stroff < stroffUpto				&&
 	       ( scc->sccCharKinds[*str] & CHARisALNUM )	)
 	    {
 	    indAddCharacterToPossibilities( &ssj, *str );
@@ -89,7 +142,7 @@ int docSpellParaFindNext(	DocumentSelection *		ds,
 		{ XDEB(maxpw); return -1;	}
 
 	    docSetParaSelection( ds, paraBi, direction,
-						maxpw->pwStartPosition,
+						maxpw->pwStartAt,
 						maxpw->pwInsertionPoint );
 	    indCleanSpellScanJob( &ssj );
 
