@@ -4,7 +4,7 @@
 /*									*/
 /************************************************************************/
 
-#   include	"config.h"
+#   include	"tedConfig.h"
 
 #   include	"appIcons.h"
 #   include	"tedApp.h"
