@@ -386,7 +386,7 @@ static void appDocForgetCopiedSelection(	APP_WIDGET		w,
 /*									*/
 /************************************************************************/
 
-APP_EVENT_HANDLER( appDocCopyPasteHandler, w, voided, event )
+APP_EVENT_HANDLER_H( appDocCopyPasteHandler, w, voided, event )
     {
     if  ( XA_MULTIPLE == None )
 	{ XA_MULTIPLE= XInternAtom( XtDisplay( w ), "MULTIPLE", False ); }
