@@ -12,20 +12,15 @@
 #   include	<appGuiBase.h>
 #   include	<appGuiResource.h>
 
-#   include	"appGeo.h"
+#   include	"geo2DInteger.h"
 #   include	"utilPs.h"
+#   include	<psFont.h>
 #   include	"utilPrinter.h"
 #   include	"utilMailContent.h"
 #   include	<appIcons.h>
 #   include	<appTool.h>
 #   include	<appDraw.h>
-
-#   if 1
-#	define	WIDGET_NAME	""
-#   else
-#	define	WIDGET_NAME	appWidgetName( __FILE__, __LINE__ )
-#   endif
-
+#   include	<appInspector.h>
 
 /************************************************************************/
 /*									*/
@@ -87,7 +82,7 @@ typedef struct AppMenuItem
     char *		amiKeyText;
 
     MenuItemType	amiItemType;
-    APP_MENU_CALLBACK	amiCallback;
+    APP_MENU_CALLBACK_T	amiCallback;
     APP_WIDGET		amiOptionWidget;
     } AppMenuItem;
 
@@ -141,6 +136,7 @@ typedef struct EditDocument
     char *			edCheckpointFilename;
     int				edFormat;
     int				edFileReadOnly;
+    unsigned int		edDocumentId;
 
     AppToplevel			edToplevel;
 
@@ -201,6 +197,7 @@ typedef struct EditDocument
     int				edTargetTypeCount;
 
     int				edMapped;	/*  Struggle with fvwm	*/
+    int				edNotYetDrawn;	/*  For FirstVisible	*/
     } EditDocument;
 
 /************************************************************************/
@@ -218,6 +215,8 @@ typedef struct EditApplication
     char *		eaApplicationName;
     char *		eaNameAndVersion;
     char *		eaReference;
+    char *		eaPlatformCompiled;
+    char *		eaHostDateCompiled;
 
     char *		eaDocumentWidgetName;
     char *		eaPageToolName;
@@ -311,7 +310,8 @@ typedef struct EditApplication
 				AppDrawingData *		add,
 				const DocumentGeometry *	defDg );
     int			(*eaFinishDocumentSetup)(
-				struct EditApplication *	ea,
+				EditDocument *			ed );
+    void		(*eaDocumentFirstVisible)(
 				EditDocument *			ed );
     int			(*eaCanSaveDocument)(
 				const void *			privateData,
@@ -326,12 +326,13 @@ typedef struct EditApplication
 				void *				privateData,
 				int				format,
 				AppDrawingData *		add );
+    void		(*eaSuggestNup)(
+				PrintGeometry *			pg,
+				void *				privateData );
     int			(*eaPrintDocument)(
-				FILE *				f,
+				SimpleOutputStream *		sos,
 				const PrintJob *		pj,
-				const PrintGeometry *		pg,
-				int				firstPage,
-				int				lastPage );
+				const PrintGeometry *		pg );
     void		(*eaDrawRectangle)(
 				APP_WIDGET			w,
 				EditDocument *			ed,
@@ -366,9 +367,9 @@ typedef struct EditApplication
 			/************************************************/
 			/*  User input on the document widget:		*/
 			/************************************************/
-    APP_EVENT_HANDLER	eaDocumentMouseHandler;
-    APP_EVENT_HANDLER	eaDocumentKeyboardHandler;
-    APP_EVENT_HANDLER	eaObserveFocus;
+    APP_EVENT_HANDLER_T	eaDocumentMouseHandler;
+    APP_EVENT_HANDLER_T	eaDocumentKeyboardHandler;
+    APP_EVENT_HANDLER_T	eaObserveFocus;
 
     APP_SCROLLBAR_CALLBACK	eaDocHorizontalScrollbarCallback;
     APP_SCROLLBAR_CALLBACK	eaDocVerticalScrollbarCallback;
@@ -410,6 +411,7 @@ typedef struct EditApplication
     PrintDestination *	eaPrintDestinations;
     int			eaPrintDestinationCount;
     int			eaDefaultPrintDestination;
+    int			eaPrintDestinationsCollected;
 
     MailContent *	eaMailContents;
     int			eaMailContentCount;
@@ -420,6 +422,8 @@ typedef struct EditApplication
 
     APP_ATOM		eaCloseAtom;
 
+    unsigned int	eaNextDocumentId;
+
     /*  2  */
 #   ifdef USE_MOTIF
     XtAppContext		eaContext;
@@ -428,48 +432,76 @@ typedef struct EditApplication
     AppToplevel			eaToplevel;
 	APP_INPUT_METHOD	eaInputMethod;
 	APP_CURSOR		eaDocumentCursor;
-    APP_WIDGET		eaMainWindow;
-    APP_WIDGET		eaMenuBar;
-    APP_WIDGET		eaFileMenu;
-    APP_WIDGET		eaFileMenuButton;
-    APP_WIDGET		eaWinMenu;
-    APP_WIDGET		eaWinMenuButton;
-    APP_WIDGET		eaHelpMenu;
-    APP_WIDGET		eaHelpMenuButton;
+    APP_WIDGET			eaMainWindow;
+    APP_WIDGET			eaMenuBar;
+    APP_WIDGET			eaFileMenu;
+    APP_WIDGET			eaFileMenuButton;
+    APP_WIDGET			eaWinMenu;
+    APP_WIDGET			eaWinMenuButton;
+    APP_WIDGET			eaHelpMenu;
+    APP_WIDGET			eaHelpMenuButton;
 
-    void *		eaFontsTool;
-    void *		eaSpellTool;
-    void *		eaFindTool;
-    void *		eaPageTool;
-    void *		eaSymbolPicker;
-    void *		eaPrintDialog;
+    void *			eaSpellTool;
+    void *			eaFindTool;
+    void *			eaPageTool;
+    void *			eaSymbolPicker;
+    void *			eaPrintDialog;
 
-    int			eaArgc;
-    char **		eaArgv;
+    int				eaArgc;
+    char **			eaArgv;
 
-    EditDocument *	eaCurrentDocument;
-    EditDocument **	eaOpenDocuments;
-    int			eaOpenCount;
-    int			eaVisibleDocumentCount;
-    int			eaMainVisibleAsAbout;
+    EditDocument *		eaCurrentDocument;
+    EditDocument **		eaOpenDocuments;
+    int				eaOpenCount;
+    int				eaVisibleDocumentCount;
+    int				eaMainVisibleAsAbout;
 
-    char *		eaAfmDirectory;
-    char *		eaGhostscriptFontmap;
-    char *		eaGhostscriptFontToXmapping;
-    int			eaGhostscriptMappingsRead;
-    char *		eaDefaultFont;
-    char *		eaRulerFont;
-    char *		eaPrintDialogFont;
-    char *		eaFaxCommand;
-    char *		eaCustomPrintCommand;
-    char *		eaCustomPrinterName;
-    char *		eaAuthor;
-    char *		eaPageNumberFormat;
-    char *		eaFocusColor;
+    char *			eaAfmDirectory;
+    char *			eaFontDirectory;
+    char *			eaGhostscriptFontmap;
+    char *			eaGhostscriptFontToXmapping;
+    int				eaGhostscriptMappingsRead;
+    char *			eaDefaultFont;
+    char *			eaRulerFont;
+    char *			eaPrintDialogFont;
+    char *			eaFaxCommand;
+    char *			eaCustomPrintCommand;
+    char *			eaCustomPrinterName;
+    char *			eaCustomPrintCommand2;
+    char *			eaCustomPrinterName2;
+    char *			eaAuthor;
+    char *			eaPageNumberFormat;
+    char *			eaFocusColor;
+
+    int				eaSupportXvCopyPaste;
+    int				eaHideSaveToOption;
+    int				eaUsePostScriptFilters;
+    int				eaUsePostScriptIndexedImages;
+    int				eaSkipEmptyPages;
+    int				eaSkipBlankPages;
+    int				eaOmitHeadersOnEmptyPages;
 
     int				eaGotPaste;
     AppFileMessageResources	eaFileMessageResources;
     char *			eaMagnificationString;
+    char *			eaSupportXvCopyPasteString;
+    char *			eaHideSaveToOptionString;
+    char *			eaUsePostScriptFiltersString;
+    char *			eaUsePostScriptIndexedImagesString;
+    char *			eaSkipEmptyPagesString;
+    char *			eaSkipBlankPagesString;
+    char *			eaOmitHeadersOnEmptyPagesString;
+
+    PostScriptFontList		eaPostScriptFontList;
+
+    void *			eaSystemProperties;
+    void *			eaUserProperties;
+
+    int				eaGotResourceTable;
+    int				eaGotFileMessageResourceTable;
+    int				eaGotApplicationResources;
+
+    APP_WIDGET			eaAppFileHideOption;
     } EditApplication;
 
 typedef int (*APP_OPEN_DOCUMENT)(	void *		through,
@@ -483,8 +515,12 @@ typedef int (*APP_OPEN_DOCUMENT)(	void *		through,
 /*									*/
 /************************************************************************/
 
-typedef int (*FindToolFind)( void * target, void * prog );
-typedef void (*FindToolReplace)( void * target, const unsigned char * guess );
+typedef int (*FindToolFind)(		void *			target );
+typedef void (*FindToolReplace)(	void *			target,
+					const unsigned char *	guess );
+typedef int (*FindToolSetPattern)(	void *			voidea,
+					const unsigned char *	pattern,
+					int			useRegex );
 
 /************************************************************************/
 /*									*/
@@ -529,65 +565,6 @@ typedef struct AppSelectionType
 #   define	AQDrespCANCEL		3
 #   define	AQDrespOK		4
 
-/************************************************************************/
-/*									*/
-/*  Facilities to make property inspectors.				*/
-/*									*/
-/************************************************************************/
-
-typedef struct InspectorSubject
-    {
-    APP_WIDGET	isPage;
-    APP_WIDGET	isMenuitem;
-    void *	isPrivate;
-    int		isEnabled;
-
-    APP_WIDGET	isPrevButton;
-    APP_WIDGET	isNextButton;
-
-    APP_WIDGET	isSelectButton;
-    APP_WIDGET	isDeleteButton;
-
-    APP_WIDGET	isInsertButton;
-    APP_WIDGET	isAppendButton;
-
-    APP_WIDGET	isRevertButton;
-    APP_WIDGET	isApplyButton;
-    } InspectorSubject;
-
-typedef struct AppInspector
-    {
-    APP_WIDGET			aiTopWidget;
-    APP_WIDGET			aiPaned;
-    APP_WIDGET			aiSeparator;
-    APP_WIDGET			aiPageParent;
-
-    AppOptionmenu		aiSubjectOptionmenu;
-
-    AppToolDestroy		aiDestroy;
-    void *			aiTarget;
-
-    int				aiSubjectCount;
-    int				aiCurrentSubject;
-    InspectorSubject		aiSubjects[1];		/*  LAST !!	*/
-    } AppInspector;
-
-typedef struct InspectorSubjectResources
-    {
-    char *		isrSubjectName;
-    char *		isrApplyToSubject;
-    char *		isrRevert;
-
-    char *		isrNextButtonText;
-    char *		isrPrevButtonText;
-
-    char *		isrSelectButtonText;
-    char *		isrDeleteButtonText;
-
-    char *		isrInsertButtonText;
-    char *		isrAppendButtonText;
-    } InspectorSubjectResources;
-
 /************************************************************************/
 /*									*/
 /*  Routine declarations.						*/
@@ -646,91 +623,40 @@ void appQuitApplication(	APP_WIDGET		option,
 				APP_WIDGET		relative,
 				EditApplication *	ea );
 
-extern void appAppFileQuit(	APP_WIDGET	option,
-				void *		voidea,
-				void *		call_data );
+extern APP_MENU_CALLBACK_H( appAppFileQuit, option, voidea, e );
 
 extern void appDocVisible(	EditApplication *	ea,
 				EditDocument *		ed,
 				int			visible );
 
-extern void appAppFileOpen(	APP_WIDGET	option,
-				void *		voidea,
-				void *		call_data );
-
-extern void appAppFileNew(	APP_WIDGET	option,
-				void *		voidea,
-				void *		call_data );
+extern APP_MENU_CALLBACK_H( appAppFileOpen, option, voidea, e );
+extern APP_MENU_CALLBACK_H( appAppFileNew, option, voidea, e );
 
 extern void appDocSetScrollbarValues(	EditDocument *	ed );
 
 extern void appMouseWheelUp(		EditDocument *	ed );
 extern void appMouseWheelDown(		EditDocument *	ed );
 
-extern void appDocFileSaveAs(	APP_WIDGET	option,
-				void *		voided,
-				void *		call_data );
-
-extern void appDocFileSaveTo(	APP_WIDGET	option,
-				void *		voided,
-				void *		call_data );
+extern APP_MENU_CALLBACK_H( appDocFileSaveAs, option, voided, call_data );
 
 extern void appSetShellConstraints(	EditDocument *		ed );
 
-extern void appAppFileMini(	APP_WIDGET	option,
-				void *		voidea,
-				void *		call_data );
+extern APP_MENU_CALLBACK_H( appAppFileMini, option, voided, call_data );
+extern APP_MENU_CALLBACK_H( appAppFileHide, option, voided, call_data );
+extern APP_MENU_CALLBACK_H( appDocFileNew, option, voided, call_data );
+extern APP_MENU_CALLBACK_H( appDocFileOpen, option, voided, call_data );
+extern APP_MENU_CALLBACK_H( appDocFileSave, option, voided, call_data );
+extern APP_MENU_CALLBACK_H( appDocFileMini, option, voided, call_data );
+extern APP_MENU_CALLBACK_H( appDocFileHide, option, voided, call_data );
+extern APP_MENU_CALLBACK_H( appDocFileClose, option, voided, call_data );
+extern APP_MENU_CALLBACK_H( appDocFileQuit, option, voided, call_data );
+extern APP_MENU_CALLBACK_H( appDocEditCopy, option, voided, call_data );
+extern APP_MENU_CALLBACK_H( appDocEditCut, option, voided, call_data );
+extern APP_MENU_CALLBACK_H( appDocEditPaste, option, voided, call_data );
+extern APP_MENU_CALLBACK_H( appDocEditSelAll, option, voided, call_data );
 
-extern void appAppFileHide(	APP_WIDGET	option,
-				void *		voidea,
-				void *		call_data );
-
-extern void appDocFileNew(	APP_WIDGET	option,
-				void *		voided,
-				void *		call_data );
-
-extern void appDocFileOpen(	APP_WIDGET	option,
-				void *		voided,
-				void *		call_data );
-
-extern void appDocFileSave(	APP_WIDGET	option,
-				void *		voided,
-				void *		call_data );
-
-extern void appDocFileMini(	APP_WIDGET	option,
-				void *		voided,
-				void *		call_data );
-
-extern void appDocFileHide(	APP_WIDGET	option,
-				void *		voided,
-				void *		call_data );
-
-extern void appDocFileClose(	APP_WIDGET	option,
-				void *		voided,
-				void *		call_data );
-
-extern void appDocFileQuit(	APP_WIDGET	option,
-				void *		voided,
-				void *		call_data );
-
-extern void appDocEditCopy(	APP_WIDGET	option,
-				void *		voided,
-				void *		voidpbcs );
-
-extern void appDocEditCut(	APP_WIDGET	option,
-				void *		voided,
-				void *		voidpbcs );
-
-extern void appDocEditPaste(	APP_WIDGET	option,
-				void *		voided,
-				void *		voidpbcs );
-
-extern void appDocEditSelAll(	APP_WIDGET	option,
-				void *		voided,
-				void *		voidpbcs );
-
-extern APP_CLOSE_CALLBACK( appDocFileCloseCallback, w, voided );
-extern APP_CLOSE_CALLBACK( appAppWmClose, w, voidea );
+extern APP_CLOSE_CALLBACK_H( appDocFileCloseCallback, w, voided );
+extern APP_CLOSE_CALLBACK_H( appAppWmClose, w, voidea );
 
 extern void appRunReallyCloseDialog(	APP_WIDGET		option,
 					EditDocument *		ed );
@@ -863,22 +789,16 @@ extern int appRunMailDialog(		EditApplication *	ea,
 
 extern int appPrintDocument(	int				printer,
 				const PrintJob *		pj,
-				const PrintGeometry *		pg,
-				int				firstPage,
-				int				lastPage );
+				const PrintGeometry *		pg );
 
 extern int appFaxDocument(	EditDocument *			ed,
 				const char *			faxNumber,
-				const PrintGeometry *		pg,
-				int				firstPage,
-				int				lastPage );
+				const PrintGeometry *		pg );
 
 extern void appDocPrintToFile(	APP_WIDGET			option,
 				APP_WIDGET			panel,
 				EditDocument *			ed,
-				const PrintGeometry *		pg,
-				int				firstPage,
-				int				lastPage );
+				const PrintGeometry *		pg );
 
 extern void appDocAbout(	APP_WIDGET	option,
 				void *		voided,
@@ -893,8 +813,8 @@ extern void appRunOpenChooser(	APP_WIDGET		option,
 				APP_OPEN_DOCUMENT	openDocument,
 				EditApplication *	ea );
 
-extern APP_EVENT_HANDLER( appDocCopyPasteHandler, w, voided, event );
-extern APP_EVENT_HANDLER( appAppGotPasteCall, w, voided, event );
+extern APP_EVENT_HANDLER_H( appDocCopyPasteHandler, w, voided, event );
+extern APP_EVENT_HANDLER_H( appAppGotPasteCall, w, voided, event );
 
 extern int appDocOwnSelection(	EditDocument *			ed,
 				const char *			selection,
@@ -917,7 +837,7 @@ extern void appDocVerticalScrollbarCallback(	APP_WIDGET	w,
 extern void appMakeButtonInRow(	APP_WIDGET *		pButton,
 				APP_WIDGET		buttonRow,
 				const char *		text,
-				APP_BUTTON_CALLBACK	callback,
+				APP_BUTTON_CALLBACK_T	callback,
 				void *			through,
 				int			position,
 				int			showAsDefault );
@@ -945,7 +865,7 @@ extern void appMakeLabelAndTextRow(	APP_WIDGET *	pRow,
 
 extern APP_WIDGET appMakeToggleInRow(	APP_WIDGET		buttonRow,
 					const char *		text,
-					APP_TOGGLE_CALLBACK	callback,
+					APP_TOGGLE_CALLBACK_T	callback,
 					void *			through,
 					int			position );
 
@@ -954,7 +874,7 @@ extern void appMakeToggleAndTextRow(	APP_WIDGET *		pRow,
 					APP_WIDGET *		pText,
 					APP_WIDGET		parent,
 					char *			labelText,
-					APP_TOGGLE_CALLBACK	callback,
+					APP_TOGGLE_CALLBACK_T	callback,
 					void *			through,
 					int			textColumns,
 					int			textEnabled );
@@ -980,7 +900,7 @@ extern void appMakeTextInColumn(	APP_WIDGET *	pText,
 
 extern void appMakeColumnToggle(	APP_WIDGET *		pToggle,
 					APP_WIDGET		column,
-					APP_TOGGLE_CALLBACK	callback,
+					APP_TOGGLE_CALLBACK_T	callback,
 					void *			through,
 					const char *		labelText,
 					int			set );
@@ -988,15 +908,15 @@ extern void appMakeColumnToggle(	APP_WIDGET *		pToggle,
 extern void appMakeColumnDrawing(	APP_WIDGET *		pButton,
 					APP_WIDGET		column,
 					APP_DRAW_BUTTON_CALLBACK exposeCallback,
-					APP_BUTTON_CALLBACK	pushedCallback,
+					APP_BUTTON_CALLBACK_T	pushedCallback,
 					void *			through,
 					int			width );
 
 extern void appMakeVerticalDialog(	AppDialog *		ad,
 					APP_WIDGET *		pPaned,
 					EditApplication *	ea,
-					APP_CLOSE_CALLBACK	closeCallback,
-					APP_DESTROY_CALLBACK	destroyCallback,
+					APP_CLOSE_CALLBACK_T	closeCallback,
+					APP_DESTROY_CALLBACK_T	destroyCallback,
 					void *			through,
 					char *			widgetName );
 
@@ -1043,11 +963,9 @@ extern void appDrawNupDiagram(	APP_WIDGET			w,
 				const DocumentGeometry *	dgPage,
 				const PrintGeometry *		pg );
 
-extern int appCallPrintFunction( FILE *				f,
+extern int appCallPrintFunction( SimpleOutputStream *		sos,
 				const PrintJob *		pj,
-				const PrintGeometry *		pg,
-				int				firstPage,
-				int				lastPage );
+				const PrintGeometry *		pg );
 
 extern void appStringToTextWidget(	APP_WIDGET		w,
 					const char *		s );
@@ -1061,27 +979,37 @@ extern void appDoubleToTextWidget(	APP_WIDGET		w,
 extern int appImgMakeFileExtensions(	AppFileExtension **	pAfeList,
 					int *			pAfeCount );
 
-extern void appInspectorSelectSubject(	AppInspector *		ai,
-					int			subject );
-
 extern void appInspectorMakeButtonRow(	APP_WIDGET *		pRow,
 					APP_WIDGET		parent,
 					APP_WIDGET *		pLeftButton,
 					APP_WIDGET *		pRightButton,
-					char *			leftLabel,
-					char *			rightLabel,
-					APP_BUTTON_CALLBACK	leftCallback,
-					APP_BUTTON_CALLBACK	rightCallback,
+					const char *		leftLabel,
+					const char *		rightLabel,
+					APP_BUTTON_CALLBACK_T	leftCallback,
+					APP_BUTTON_CALLBACK_T	rightCallback,
+					void *			through );
+
+extern void appInspectorMake3ButtonRow( APP_WIDGET *		pRow,
+					APP_WIDGET		parent,
+					APP_WIDGET *		pLeftButton,
+					APP_WIDGET *		pMiddleButton,
+					APP_WIDGET *		pRightButton,
+					const char *		leftLabel,
+					const char *		middleLabel,
+					const char *		rightLabel,
+					APP_BUTTON_CALLBACK_T	leftCallback,
+					APP_BUTTON_CALLBACK_T	middleCallback,
+					APP_BUTTON_CALLBACK_T	rightCallback,
 					void *			through );
 
 extern void appInspectorMakeToggleRow(	APP_WIDGET *		pRow,
 					APP_WIDGET		parent,
 					APP_WIDGET *		pLeftToggle,
 					APP_WIDGET *		pRightToggle,
-					char *			leftText,
-					char *			rightText,
-					APP_TOGGLE_CALLBACK	leftCallback,
-					APP_TOGGLE_CALLBACK	rightCallback,
+					const char *		leftText,
+					const char *		rightText,
+					APP_TOGGLE_CALLBACK_T	leftCallback,
+					APP_TOGGLE_CALLBACK_T	rightCallback,
 					void *			through );
 
 extern AppInspector * 	appMakeInspector(
@@ -1094,15 +1022,6 @@ extern AppInspector * 	appMakeInspector(
 				    AppToolDestroy		closeInspector,
 				    void *			through );
 
-extern void appFinishInspector(		AppInspector *		ai );
-
-extern void appEnableInspector(		AppInspector *		ai,
-					int			enabled );
-
-extern void appEnableInspectorSubject(	AppInspector *		ai,
-					int			subject ,
-					int			enabled );
-
 extern void appInspectorMakeMenuRow(	APP_WIDGET *		pRow,
 					AppOptionmenu *		aom,
 					APP_WIDGET *		pLabel,
@@ -1114,13 +1033,13 @@ extern void appFillInspectorMenu(	int			count,
 					APP_WIDGET *		items,
 					char * const *		texts,
 					AppOptionmenu *		aom,
-					APP_BUTTON_CALLBACK	callBack,
+					APP_OITEM_CALLBACK_T	callBack,
 					void *			target );
 
 extern int appMakeDocumentWidget(	EditApplication *	ea,
 					EditDocument *		ed );
 
-extern APP_EVENT_HANDLER( appDocExposeHandler, w, d, exposeEvent );
+extern APP_EVENT_HANDLER_H( appDocExposeHandler, w, d, exposeEvent );
 
 extern int appFileConvert(	EditApplication *	ea,
 				const char *		fromName,
@@ -1136,14 +1055,12 @@ extern int appPrintToPrinter(	EditApplication *	ea,
 				const char *		toName,
 				const char *		paperString );
 
-extern void appDestroyEditDocument(	APP_WIDGET		w,
-					void *			voided,
-					void *			callData );
+APP_DESTROY_CALLBACK_H( appDestroyEditDocument, w, voided );
 
 extern APP_WIDGET appMakePageDrawing(	APP_WIDGET		parent,
 					EditApplication *	ea,
 					int			mmHigh,
-					APP_EVENT_HANDLER	redraw,
+					APP_EVENT_HANDLER_T	redraw,
 					void *			through );
 
 extern void appScrollToRectangle(	EditDocument *		ed,
@@ -1162,7 +1079,7 @@ extern void appMakeVerticalTool( APP_WIDGET *		pShell,
 				const char *		widgetName,
 				int			userResizable,
 				APP_WIDGET		option,
-				APP_CLOSE_CALLBACK	closeCallback,
+				APP_CLOSE_CALLBACK_T	closeCallback,
 				void *			through );
 
 extern APP_WIDGET appMakeRowInColumn(	APP_WIDGET	parent,
@@ -1178,19 +1095,26 @@ extern void * appMakeFindTool(		APP_WIDGET		findOption,
 					FindToolFind		findNext,
 					FindToolFind		findPrev,
 					FindToolReplace		replace,
+					FindToolSetPattern	setPattern,
 					void *			target );
 
+extern void appFindToolSetPattern(	void *			voidaft,
+					const unsigned char *	pattern,
+					int			useRegex );
+
 extern void appShowFindTool(		APP_WIDGET		reference,
 					void *			voidaft );
 
-extern void appEnableFindTool(		void *	voidaft,
-					int	enabled );
+extern void appEnableFindTool(		void *		voidaft,
+					int		enabled );
 
-extern void appFindToolDisableReplace(	void *	voidaft );
+extern void appFindToolEnableReplace(	void *		voidaft,
+					int		enabled );
 
 extern void appMakeColumnInRow(		APP_WIDGET *	pColumn,
 					APP_WIDGET	row,
-					int		position );
+					int		position,
+					int		colspan );
 
 extern void appMakeRowFrameInColumn(	APP_WIDGET *	pFrame,
 					APP_WIDGET *	pRow,
@@ -1204,11 +1128,18 @@ extern void	appFreeStringFromTextWidget(	char *		s );
 extern char *	appGetTextFromMenuOption(	APP_WIDGET	w );
 extern void	appFreeTextFromMenuOption(	char *		s );
 
-extern void appGuiGetResourceValues(	EditApplication *	ea,
+extern void appGuiGetResourceValues(
+				int *				pGotResources,
+				EditApplication *		ea,
 				void *				pValues,
 				AppConfigurableResource *	acr,
 				int				acrCount );
 
+extern void appSetResourceDefaults(
+				EditApplication *		ea,
+				AppConfigurableResource *	acr,
+				int				acrCount );
+
 extern int appCountChangedDocuments(	EditApplication *	ea );
 
 extern int appGuiInitApplication(	EditApplication *	ea,
@@ -1245,6 +1176,9 @@ extern void appSetOptionmenu(		AppOptionmenu *		aom,
 extern void appGuiEnableWidget(		APP_WIDGET		w,
 					int			on_off );
 
+extern void appGuiSetWidgetVisible(	APP_WIDGET		w,
+					int			on_off );
+
 extern void appShowShellWidget(		APP_WIDGET		shell );
 extern void appHideShellWidget(		APP_WIDGET		shell );
 extern void appDestroyShellWidget(	APP_WIDGET		shell );
@@ -1266,26 +1200,29 @@ extern void appGuiEnableOptionmenu(	AppOptionmenu *		aom,
 
 extern APP_WIDGET appAddItemToOptionmenu( AppOptionmenu *	aom,
 					const char *		label,
-					APP_BUTTON_CALLBACK	callBack,
-					void *			target );
-
-extern APP_WIDGET appAddPixmapItemToOptionmenu( AppOptionmenu *	aom,
-					APP_BUTTON_CALLBACK	callBack,
+					APP_OITEM_CALLBACK_T	callBack,
 					void *			target );
 
 extern void appSetShellTitle(		APP_WIDGET		shell,
 					APP_WIDGET		option,
-					const char *		appName );
+					const char *		title );
+
+extern void appGuiLowerShellWidget(	APP_WIDGET		shell );
 
 /************************************************************************/
+/*									*/
 /*  Event handler to set minimum size.					*/
+/*									*/
 /************************************************************************/
 
-extern APP_EVENT_HANDLER( appSetSizeAsMinimum, w, through, event );
+extern APP_EVENT_HANDLER_H( appSetSizeAsMinimum, w, through, event );
 
 /************************************************************************/
+/*									*/
 /*  Names of the X11 events.						*/
+/*									*/
 /************************************************************************/
+
 extern char * APP_X11EventNames[];
 
 extern void appGuiChangeLabelText(	APP_WIDGET	labelWidget,
@@ -1313,25 +1250,25 @@ extern void appMakeOptionmenuInRow(	AppOptionmenu *	aom,
 					int		column,
 					int		colspan );
 
-extern int appGuiGetToggleStateFromCallback( APP_WIDGET		toggle,
-					void *			voidcbs );
-
-extern int appGuiGetMenuToggleStateFromCallback( APP_WIDGET		toggle,
+extern int appGuiGetMenuToggleStateFromCallback(
+					APP_WIDGET		toggle,
 					void *			voidcbs );
 
 extern void appGuiMakeListInColumn(	APP_WIDGET *		pList,
 					APP_WIDGET		column,
 					int			visibleItems,
-					APP_CALLBACK_FUNC	callback,
+					APP_LIST_CALLBACK_T	callback,
 					void *			through );
 
-extern void appGuiSetTypingCallbackForText( APP_WIDGET		text,
-					APP_CALLBACK_FUNC	callBack,
-					void *		through );
+extern void appGuiSetTypingCallbackForText(
+					APP_WIDGET		text,
+					APP_TXTYPING_CALLBACK_T	callBack,
+					void *			through );
 
-extern void appGuiSetGotValueCallbackForText( APP_WIDGET	text,
-					APP_CALLBACK_FUNC	callBack,
-					void *		through );
+extern void appGuiSetGotValueCallbackForText(
+				    APP_WIDGET			text,
+				    APP_TXACTIVATE_CALLBACK_T	callBack,
+				    void *			through );
 
 extern void appGuiGetScrollbarValues(	int *		pValue,
 					int *		pSliderSize,
@@ -1372,9 +1309,6 @@ extern void appGuiSelectPositionInListWidget(	APP_WIDGET	list,
 
 extern void appGuiRemoveSelectionFromListWidget(	APP_WIDGET	list );
 
-extern int appGuiGetPositionFromListCallback(	APP_WIDGET	list,
-						void *		voidlcs );
-
 extern APP_WIDGET appSetMenuItem(	APP_WIDGET		menu,
 					AppToplevel *		at,
 					AppMenuItem *		ami,
@@ -1396,8 +1330,9 @@ extern APP_WIDGET appMakeMenuInParent(	APP_WIDGET *		pButton,
 					const char *		itemText,
 					int			isHelp );
 
-extern char * appGuiGetStringFromListCallback(	APP_WIDGET	list,
-						void *		voidlcs );
+extern char * appGuiGetStringFromListCallback(
+					    APP_LIST		list,
+					    APP_LIST_CHOICE *	voidlcs );
 
 extern void appFreeStringFromListCallback(	char *		s );
 
@@ -1426,7 +1361,7 @@ extern void appGuiMakeDrawingAreaInColumn( APP_WIDGET *		pDrawing,
 				int				wide,
 				int				high,
 				int				heightResizable,
-				APP_EVENT_HANDLER		redraw,
+				APP_EVENT_HANDLER_T		redraw,
 				void *				through );
 
 void appGuiGetStringFromKeyboardEvent(	APP_INPUT_CONTEXT	ic,
@@ -1435,7 +1370,7 @@ void appGuiGetStringFromKeyboardEvent(	APP_INPUT_CONTEXT	ic,
 					int *			pGotString,
 					int *			pGotKey,
 					unsigned int *		pState,
-					char *			buf,
+					unsigned char *		buf,
 					int			capacity,
 					APP_KEY_VALUE *		pKey );
 
@@ -1450,26 +1385,26 @@ extern void appInitOptionmenu(		AppOptionmenu *		aom );
 extern void appRunDragLoop(	APP_WIDGET		w,
 				EditApplication *	ea,
 				APP_EVENT *		downEvent,
-				APP_EVENT_HANDLER	upHandler,
-				APP_EVENT_HANDLER	moveHandler,
+				APP_EVENT_HANDLER_T	upHandler,
+				APP_EVENT_HANDLER_T	moveHandler,
 				int			timerInterval,
 				APP_TIMER_CALLBACK	timerHandler,
 				void *			through );
 
 extern void appGuiSetFocusChangeHandler( APP_WIDGET		shell,
-					APP_EVENT_HANDLER	handler,
+					APP_EVENT_HANDLER_T	handler,
 					void *			through );
 
 extern void appGuiSetToggleItemState(	APP_WIDGET		toggle,
 					int			set );
 
 extern void appSetDestroyCallback(	APP_WIDGET		shell,
-					APP_DESTROY_CALLBACK	destroyCallback,
+					APP_DESTROY_CALLBACK_T	destroyCallback,
 					void *			through );
 
 extern void appSetCloseCallback(	APP_WIDGET		shell,
 					EditApplication *	ea,
-					APP_CLOSE_CALLBACK	closeCallback,
+					APP_CLOSE_CALLBACK_T	closeCallback,
 					void *			through );
 
 extern void appGuiSetToggleItemLabel(	APP_WIDGET		toggle,
@@ -1481,12 +1416,18 @@ extern int appGuiGtkGetChildLabel(	char **			pLabel,
 extern int appGuiGtkSetChildLabel(	APP_WIDGET		w,
 					const char *		s );
 
+extern void appGuiGetResourceValuesGtkX(
+				EditApplication *		ea,
+				void *				pValues,
+				AppConfigurableResource *	acr,
+				int				acrCount );
+
 extern int appFormatDocumentTitle(	const char **		pWindowTitle,
 					const char **		pIconTitle,
 					EditApplication *	ea,
 					const char *		title );
 
-extern APP_EVENT_HANDLER( appDocConfigure, w, voided, event );
+extern APP_EVENT_HANDLER_H( appDocConfigure, w, voided, event );
 
 extern void appDocumentCalculateExtraSize(	EditDocument *	ed );
 
@@ -1508,8 +1449,6 @@ extern APP_GIVE_COPY( appDocReplyToCopyRequest, w, gsd, voided );
 extern void appCopyPixmapValue(	APP_SELECTION_EVENT *	event,
 				APP_BITMAP_IMAGE	pixmapCopied );
 
-extern void appPixmapOptionmenuSetButtonMargins(	AppOptionmenu *	aom );
-
 extern void appGetApplicationResourceValues(	EditApplication *	ea );
 
 extern void appDocExposeRectangle(
@@ -1521,10 +1460,95 @@ extern void appDocExposeRectangle(
 extern void appPrintJobForEditDocument(	PrintJob *		pj,
 					EditDocument *		ed );
 
+extern void appApplicationSettingsToPrintGeometry(
+					PrintGeometry *		pg,
+					EditApplication *	ea );
+
 extern int appRunPrintToFileChooser(	APP_WIDGET		option,
 					APP_WIDGET		panel,
 					EditApplication *	ea,
 					const EditDocument *	ed,
 					char **			pFilename );
 
+extern int appDocumentTestCanSave(
+				EditApplication *		ea,
+				const AppFileExtension *	afe,
+				void *				privateData,
+				unsigned int			useFlags,
+				int				format );
+
+extern int appDocumentGetSaveFormat(
+				EditApplication *		ea,
+				const char *			filename,
+				void *				privateData,
+				unsigned int			useFlags,
+				int				format );
+
+extern void appMakeDrawnPulldownInColumn(
+				AppDrawnPulldown *		adp,
+				APP_EVENT_HANDLER_T		redrawInplace,
+				APP_EVENT_HANDLER_T		redrawPulldown,
+				APP_EVENT_HANDLER_T		clickHandler,
+				APP_WIDGET			column,
+				void *				through );
+
+extern void appMakeDrawnPulldownInRow(
+				AppDrawnPulldown *		adp,
+				APP_EVENT_HANDLER_T		redrawInplace,
+				APP_EVENT_HANDLER_T		redrawPulldown,
+				APP_EVENT_HANDLER_T		clickHandler,
+				APP_WIDGET			row,
+				int				column,
+				int				colspan,
+				void *				through );
+
+extern void appGuiEnableDrawnPulldown(	AppDrawnPulldown *	adp,
+					int			sensitive );
+
+extern void appInitDrawnPulldown(	AppDrawnPulldown *	adp );
+
+extern void appGuiSetDrawnPulldownHeight(
+				    AppDrawnPulldown *	adp,
+				    int			height );
+
+extern void appGuiSetDrawnPulldownStrips(
+				    AppDrawnPulldown *	adp,
+				    int			strips );
+
+extern int appGuiDrawnPulldownGetStrip(
+				int *				pStrip,
+				const AppDrawnPulldown *	adp,
+				const APP_WIDGET		w,
+				const APP_EVENT *		mouseEvent );
+
+extern void appDrawnPulldownDrawArrow(	int *			pRestWide,
+					int *			pRestHigh,
+					APP_WIDGET		w,
+					AppDrawingData *	add );
+
+extern APP_FONT * appGuiGetLabelFont(	APP_WIDGET	w );
+
+extern int appGuiGetPositionFromListCallback(	APP_LIST	list,
+						void *		voidlcs );
+
+extern void appOptionmenuItemSetVisibility(	APP_WIDGET	w,
+						int		visible );
+
+extern int appPostScriptFontCatalog(		EditApplication *	ea );
+
+extern int appGetPrintDestinations(		EditApplication *	ea );
+
+extern int appReadSystemProperties(	EditApplication *	ea );
+extern int appReadUserProperties(	EditApplication *	ea );
+extern int appSetUserProperty(		EditApplication *	ea,
+					const char *		name,
+					const char *		value );
+
+extern int appSetSystemProperty(	EditApplication *	ea,
+					const char *		name,
+					const char *		value );
+
+extern int appGuiSetFrameTitle(		APP_WIDGET		w,
+					const unsigned char *	title );
+
 #   endif
