@@ -7,14 +7,14 @@
 #   ifndef	APP_ICONS_H
 #   define	APP_ICONS_H
 
-#   include	<X11/Xlib.h>
+#   include	<appGuiBase.h>
 
 typedef struct NamedPicture
     {
-    char *	npPictureName;
-    char **	npBuffer;
-    Pixmap	npPixmap;
-    Pixmap	npShapemask;
+    char *		npPictureName;
+    char **		npBuffer;
+    APP_BITMAP_IMAGE	npPixmap;
+    APP_BITMAP_MASK	npMask;
     } NamedPicture;
 
 #   endif
