@@ -5,13 +5,13 @@
 /************************************************************************/
 
 #   include	<appFrame.h>
+#   include	<docExpandedTextAttribute.h>
 
-typedef void (*SymbolPickerInsert)(	void *		target,
-					int		symbol,
-					int		fontFamilyNumber,
-					char *		fontFamilyName,
-					int		encoding,
-					int		encodingIsDefault );
+typedef void (*SymbolPickerInsert)(
+				void *				target,
+				int				symbol,
+				const TextAttribute *		taSet,
+				const PropertyMask *		taSetMask );
 
 extern void * appMakeSymbolPicker(	APP_WIDGET		symbolOption,
 					EditApplication *	ea,
@@ -24,9 +24,12 @@ extern void * appMakeSymbolPicker(	APP_WIDGET		symbolOption,
 
 extern void appShowSymbolPicker(	void *	voidasp	);
 
-extern int appAdaptSymbolPickerToFontFamily(	void *	voidasp,
-						int	fontFamilyNumber,
-						int	encoding );
+extern int appAdaptSymbolPickerToFontFamily(
+				void *				voidasp,
+				unsigned int			documentId,
+				const DocumentFontList *	dfl,
+				const TextAttribute *		taSet,
+				const PropertyMask *		setMask );
 
-extern void appEnableSymbolPicker(	void *	voidasp,
-					int	enabled );
+extern void appEnableSymbolPicker(	void *			voidasp,
+					int			enabled );
