@@ -340,53 +340,6 @@ int appMakeUniqueString(	char *		target,
     return 0;
     }
 
-/************************************************************************/
-/*									*/
-/*  Translate a file name to an absolute name.				*/
-/*									*/
-/************************************************************************/
-
-int appAbsoluteName(	char *		absolute,
-			int		len,
-			const char *	filename )
-    {
-    int		rellen;
-    int		abslen;
-
-    rellen= strlen( filename );
-
-    if  ( filename[0] == '/' )
-	{
-	if  ( rellen > len )
-	    { SLDEB(filename,len); return -1;	}
-
-	strcpy( absolute, filename ); return rellen;
-	}
-
-    if  ( rellen > len- 2 )
-	{ SLDEB(filename,len); return -1;	}
-
-    abslen= appCurrentDirectory( absolute, len- rellen- 2 );
-    if  ( abslen < 1 )
-	{ LSDEB(abslen,filename); return -1;	}
-
-#   ifdef __VMS
-
-    strcpy( absolute+ abslen, filename );
-
-    return abslen+ rellen;
-
-#   else
-
-    absolute[abslen]= '/';
-
-    strcpy( absolute+ abslen+ 1, filename );
-
-    return abslen+ 1+ rellen;
-
-#   endif
-    }
-
 /************************************************************************/
 /*									*/
 /*  Remove a file.							*/
@@ -609,7 +562,7 @@ int appOpenSocket(	const char *		hostName,
 int appListenSocket(		const char *	portName,
 				unsigned int *	pPort,
 				void *		through,
-				APP_COMPLAIN	complain	)
+				APP_COMPLAIN	complain )
     {
     struct sockaddr_in	si;
     unsigned int	sinlen= sizeof( struct sockaddr_in );
@@ -620,7 +573,7 @@ int appListenSocket(		const char *	portName,
     memset( &si, 0, sinlen );
 
     c= 0;
-    if  ( sscanf( portName, "%d%c", &port, &c ) == 1 && c == 0 )
+    if  ( sscanf( portName, "%u%c", &port, &c ) == 1 && c == 0 )
 	{ si.sin_port= htons( port ); }
     else{
 	struct servent *	serv= getservbyname( portName, "tcp" );
