@@ -1,6 +1,7 @@
 /************************************************************************/
 /*									*/
-/*  Simple io streams using the C stdio.				*/
+/*  Simple io streams, Data on the stream travels in hexadecimal	*/
+/*  format.								*/
 /*									*/
 /************************************************************************/
 
@@ -8,4 +9,9 @@
 
 extern SimpleInputStream * sioInHexOpen(	SimpleInputStream * sisHex );
 
+extern SimpleOutputStream * sioOutHexOpenFolded(
+				    SimpleOutputStream *	sosHex,
+				    int				wide,
+				    int				lastNl );
+
 extern SimpleOutputStream * sioOutHexOpen(	SimpleOutputStream * sosHex );
