@@ -8,17 +8,9 @@
 #   define	SIO_MEMORY_H
 
 #   include	<sioGeneral.h>
+#   include	<utilMemoryBuffer.h>
 
-typedef struct MemoryBuffer
-    {
-    int			mbCapacity;
-    unsigned char *	mbBytes;
-    } MemoryBuffer;
-
-extern void sioMemoryInit(	MemoryBuffer *	mb	);
-extern void sioMemoryClean(	MemoryBuffer *	mb	);
-
-extern SimpleInputStream * sioInMemoryOpen(	MemoryBuffer *	mb );
+extern SimpleInputStream * sioInMemoryOpen(	const MemoryBuffer *	mb );
 
 extern SimpleOutputStream * sioOutMemoryOpen(	MemoryBuffer *	mb );
 
