@@ -1,13 +1,16 @@
 /************************************************************************/
 /*									*/
 /*  sioSmtp.[ch]:	Send an e-mail message using the SMTP protocol.	*/
-/*  Author:		Robert A. Vonk (rvonk@knoware.nl)		*/
+/*  Author:		Robert A. Vonk.					*/
 /*			( Adapted by Mark de Does to fit in the sio	*/
-/*			framework. )					*/
+/*			framework. Also added elementary ESMTP AUTH	*/
+/*			support.					*/
 /*									*/
 /*									*/
 /*  SimpleOutputStream * sioOutSmtpOpen(char *	mailHost,		*/
 /*					char *	mailPort,		*/
+/*					char *	user,			*/
+/*					char *	pass,			*/
 /*					char *	From,			*/
 /*					char *	To,			*/
 /*					char *	Cc,			*/
@@ -20,6 +23,13 @@
 /*			DEFAULT: "mailhost".				*/
 /*	  mailPort:	The name of the service port for smtp.		*/
 /*			DEFAULT: "smtp".				*/
+/*	  user:		If not null, this is the user name to log in	*/
+/*			into the smtp server. Only some AUTH variants	*/
+/*			are supported.					*/
+/*	  pass:		If not null, this is the password to log in	*/
+/*			into the smtp server. Only some AUTH variants	*/
+/*			are supported.					*/
+/*			DEFAULT: no default: You must give a non empty	*/
 /*	  From:		The e-mail address of the sender.		*/
 /*			DEFAULT: no default: You must give a non empty	*/
 /*			value.						*/
@@ -40,6 +50,10 @@
 /*	  mimeBoundary:	The MIME boundary used to separate parts of the	*/
 /*			message.					*/
 /*			DEFAULT: No mime boundary.			*/
+/*	  through:	Pass through pointer to the error routine.	*/
+/*	  complain:	Called to pass error information back to the	*/
+/*			user. Only called when things go wrong.		*/
+/*									*/
 /*	RETURN VALUES:							*/
 /*	  (SimpleOutputStream *)0:	On failure.			*/
 /*	  (SimpleOutputStream *)ptr:	On success.			*/
@@ -55,6 +69,8 @@
 
 extern SimpleOutputStream * sioOutSmtpOpen(	const char *	mailHost,
 						const char *	mailPort,
+						const char *	user,
+						const char *	pass,
 						const char *	from,
 						const char *	to,
 						const char *	cc,
