@@ -1,5 +1,7 @@
 /************************************************************************/
+/*									*/
 /*  A mail content type.						*/
+/*									*/
 /************************************************************************/
 
 #   ifndef	UTIL_MAIL_CONTENT_H
@@ -8,14 +10,15 @@
 #   include	<sioGeneral.h>
 
 typedef int (*WriteMail)(	SimpleOutputStream *	sos,
+				const char *		mimeBoundary,
 				void *			voiddoc );
 
 typedef struct MailContent
     {
     char *		mcResourceName;
     char *		mcNameText;
-    char *		mcType;
-    char *		mcSubtype;
+    char *		mcTypeSlashSubtype;
+    int			mcNeedsBoundary;
     WriteMail		mcWriteMail;
     } MailContent;
 
