@@ -52,7 +52,7 @@ const char *	bmunStrings[]=
     0
     };
 
-const char *	bmunIntToString( int unitInt )
+const char * bmunIntToString( int unitInt )
     {
     if  ( unitInt >= 0						&&
 	  unitInt < sizeof( bmcoStrings )/sizeof(char *) -1	)
@@ -61,7 +61,7 @@ const char *	bmunIntToString( int unitInt )
     LDEB(unitInt); return (char *)0;
     }
 
-int		bmunStringToInt( const char * unitString )
+int bmunStringToInt( const char * unitString )
     {
     int	i;
 
@@ -75,8 +75,11 @@ int		bmunStringToInt( const char * unitString )
     }
 
 /************************************************************************/
+/*									*/
 /*  Initialise a BitmapDescription.					*/
+/*									*/
 /************************************************************************/
+
 void bmInitDescription	( BitmapDescription *	bd )
     {
     bd->bdBufferLength= 0;
@@ -214,3 +217,48 @@ void bmImageSizeTwips(	int *				pImageWideTwips,
     return;
     }
 
+/************************************************************************/
+/*									*/
+/*  Derive the sizes of a description from the other fields.		*/
+/*									*/
+/************************************************************************/
+
+int bmCalculateSizes(	BitmapDescription *	bd )
+    {
+    switch( bd->bdColorEncoding )
+	{
+	case BMcoBLACKWHITE:
+	case BMcoWHITEBLACK:
+	    if  ( bd->bdHasAlpha )
+		{ bd->bdSamplesPerPixel= 2;	}
+	    else{ bd->bdSamplesPerPixel= 1;	}
+
+	    bd->bdBitsPerPixel= bd->bdSamplesPerPixel* bd->bdBitsPerSample;
+	    break;
+
+	case BMcoRGB:
+	    if  ( bd->bdHasAlpha )
+		{ bd->bdSamplesPerPixel= 4;	}
+	    else{ bd->bdSamplesPerPixel= 3;	}
+
+	    bd->bdBitsPerPixel= bd->bdSamplesPerPixel* bd->bdBitsPerSample;
+	    break;
+
+	case BMcoRGB8PALETTE:
+	    if  ( bd->bdHasAlpha )
+		{ bd->bdSamplesPerPixel= 4;	}
+	    else{ bd->bdSamplesPerPixel= 3;	}
+
+	    break;
+
+	default:
+	    LDEB(bd->bdColorEncoding);
+	    return -1;
+	}
+
+    bd->bdBytesPerRow= ( bd->bdBitsPerPixel* bd->bdPixelsWide+ 7 )/ 8;
+    bd->bdBufferLength= bd->bdPixelsHigh* bd->bdBytesPerRow;
+
+    return 0;
+    }
+
