@@ -187,8 +187,8 @@ int bmWhiteToTransparent(	BitmapDescription *		bdOut,
 	    LDEB(bdIn->bdColorEncoding); return -1;
 	}
 
-    bd.bdBytesPerRow= ( bd.bdPixelsWide* bd.bdBitsPerPixel+ 7 )/8;
-    bd.bdBufferLength= bd.bdBytesPerRow* bd.bdPixelsHigh;
+    if  ( bmCalculateSizes( &bd ) )
+	{ LDEB(1); return -1;	}
     bufOut= malloc( bd.bdBufferLength );
     if  ( ! bufOut )
 	{ LLDEB(bd.bdBufferLength,bufOut); return -1;	}
@@ -225,86 +225,3 @@ int bmWhiteToTransparent(	BitmapDescription *		bdOut,
     return 0;
     }
 
-/************************************************************************/
-/*									*/
-/*  Make a completely transparent image					*/
-/*									*/
-/************************************************************************/
-
-int bmTransparentImage(		BitmapDescription *		bdOut,
-				unsigned char **		pBufOut,
-				int				colorEncoding,
-				int				wide,
-				int				high )
-    {
-    BitmapDescription		bd;
-    unsigned char *		buffer;
-
-    switch( colorEncoding )
-	{
-	case BMcoRGB8PALETTE:
-	    bd.bdPixelsWide= wide;
-	    bd.bdPixelsHigh= high;
-	    bd.bdHasAlpha= 1;
-	    bd.bdUnit= BMunINCH;
-	    bd.bdXResolution= 72;
-	    bd.bdYResolution= 72;
-
-	    bd.bdBitsPerSample= 8;
-	    bd.bdSamplesPerPixel= 3;
-	    bd.bdBitsPerPixel= 4;
-	    bd.bdColorEncoding= BMcoRGB8PALETTE;
-	    bd.bdColorCount= 3;
-	    bd.bdRGB8Palette= (RGB8Color *)0;
-	    bd.bdBytesPerRow= bd.bdPixelsWide;
-	    bd.bdBufferLength= bd.bdPixelsHigh* bd.bdBytesPerRow;
-
-	    if  ( bd.bdHasAlpha )
-		{
-		bd.bdSamplesPerPixel= 4;
-		bd.bdBitsPerPixel *= 2;
-		bd.bdBytesPerRow *= 2;
-		bd.bdBufferLength *= 2;
-		}
-
-	    buffer= (unsigned char *)malloc( bd.bdBufferLength );
-	    if  ( ! buffer )
-		{ LXDEB(bd.bdBufferLength,buffer); return -1; }
-
-	    bd.bdRGB8Palette= (RGB8Color *)
-				    malloc( 256* sizeof( RGB8Color ) );
-	    if  ( ! bd.bdRGB8Palette )
-		{
-		LLDEB(256,bd.bdRGB8Palette);
-		free( buffer ); return -1;
-		}
-
-	    /*  transparent */
-	    bd.bdRGB8Palette[0].rgb8Red= 255;
-	    bd.bdRGB8Palette[0].rgb8Green= 255;
-	    bd.bdRGB8Palette[0].rgb8Blue= 255;
-	    bd.bdRGB8Palette[0].rgb8Alpha= 0;
-
-	    /*  white */
-	    bd.bdRGB8Palette[1].rgb8Red= 255;
-	    bd.bdRGB8Palette[1].rgb8Green= 255;
-	    bd.bdRGB8Palette[1].rgb8Blue= 255;
-	    bd.bdRGB8Palette[1].rgb8Alpha= 255;
-
-	    /*  black */
-	    bd.bdRGB8Palette[2].rgb8Red= 0;
-	    bd.bdRGB8Palette[2].rgb8Green= 0;
-	    bd.bdRGB8Palette[2].rgb8Blue= 0;
-	    bd.bdRGB8Palette[2].rgb8Alpha= 0;
-
-	    memset( buffer, 0, bd.bdBufferLength );
-
-	    break;
-
-	default:
-	    LDEB(colorEncoding); return -1;
-	}
-
-    *bdOut= bd; *pBufOut= buffer;
-    return 0;
-    }
