@@ -218,28 +218,50 @@ static BitmapFileType	bmSgiFile=
 
 BitmapFileType * bmFileTypes[]=
     {
-    &bmTiffFile,
-    &bmTifFile,
-    &bmBmpFile,
-    &bmIcoFile,
-    &bmXwdFile,
-    &bmGifFile,
-    &bmEpsFile,
+    &bmPngFile,
     &bmJpegFile,
     &bmJpgFile,
+    &bmBmpFile,
+    &bmGifFile,
+    &bmXwdFile,
+    &bmEpsFile,
     &bmXbmFile,
-    &bmPngFile,
-    &bmWmfFile,
+    &bmIcoFile,
+    &bmTiffFile,
+    &bmTifFile,
     &bmPcxFile,
     &bmXpmFile,
     &bmPgmFile,
     &bmPbmFile,
     &bmPpmFile,
     &bmWbmpFile,
+    &bmWmfFile,
     };
 
 BitmapFileFormat	bmFileFormats[]=
     {
+	{ "PNG  Portable Network Graphics", "png1File",
+			    0,				&bmPngFile },
+	{ "JPG  Independent JPEG Group (.jpg)", "jpg1File",
+			    1,				&bmJpgFile },
+	{ "JPEG  Independent JPEG Group (.jpeg)", "jpeg1File",
+			    1,				&bmJpegFile },
+	{ "BMP  Microsoft Windows 3.x bitmap", "bmp3File",
+			    40,				&bmBmpFile },
+	{ "GIF  Compuserve GIF", "gif87File",
+			    87,				&bmGifFile },
+	{ "XWD  X/11 Window dump", "xwd11File",
+			    11,				&bmXwdFile },
+	{ "XWD  X/10 Window dump", "xwd10File",
+			    10,				&bmXwdFile },
+	{ "EPS  Encapsulated Postscript", "eps1File",
+			    1,				&bmEpsFile },
+	{ "EPS  Encapsulated Postscript (Level 2)", "eps2File",
+			    2,				&bmEpsFile },
+	{ "XBM  X/11 Bitmap File", "xbm11File",
+			    11,				&bmXbmFile },
+	{ "ICO  Microsoft 3.x Windows Icon", "ico3File",
+			    40,				&bmIcoFile },
 	{ "TIFF  No compression", "tiffPlainFile",
 			    COMPRESSION_NONE,		&bmTiffFile },
 	{ "TIFF  CCITT run length", "tiffCcittRleFile",
@@ -260,26 +282,6 @@ BitmapFileFormat	bmFileFormats[]=
 			    COMPRESSION_THUNDERSCAN,	&bmTiffFile },
 	{ "TIFF  JPEG", "tiffJpegFile",
 			    COMPRESSION_JPEG,		&bmTiffFile },
-	{ "BMP  Microsoft Windows 3.x bitmap", "bmp3File",
-			    40,				&bmBmpFile },
-	{ "XWD  X/11 Window dump", "xwd11File",
-			    11,				&bmXwdFile },
-	{ "XWD  X/10 Window dump", "xwd10File",
-			    10,				&bmXwdFile },
-	{ "GIF  Compuserve GIF", "gif87File",
-			    87,				&bmGifFile },
-	{ "EPS  Encapsulated Postscript", "eps1File",
-			    1,				&bmEpsFile },
-	{ "JPG  Independent JPEG Group (.jpg)", "jpg1File",
-			    1,				&bmJpgFile },
-	{ "JPEG  Independent JPEG Group (.jpeg)", "jpeg1File",
-			    1,				&bmJpegFile },
-	{ "XBM  X/11 Bitmap File", "xbm11File",
-			    11,				&bmXbmFile },
-	{ "PNG  Portable Network Graphics", "png1File",
-			    0,				&bmPngFile },
-	{ "ICO  Microsoft 3.x Windows Icon", "ico3File",
-			    40,				&bmIcoFile },
 	{ "PCX  Zsoft PC Paintbrush file", "pcx1File",
 			    0,				&bmPcxFile },
 	{ "XPM  X11 Pixmap File", "xpm1File",
