@@ -17,6 +17,8 @@
 
 #   include	"bm_gif_lib.h"
 
+int _GifError = 0;
+
 /************************************************************************/
 /*									*/
 /*  Read a GIF file.							*/
@@ -53,7 +55,7 @@ int bmGifReadGif(		BitmapDescription *	bd,
 	const RGB8Color *	rgb8From;
 	RGB8Color *		rgb8To;
 
-	if  ( DGifGetRecordType( gft, &grt ) != GIF_OK )
+	if  ( bmGifGetRecordType( gft, &grt ) != GIF_OK )
 	    { LDEB(1); DGifCloseFile( gft ); return -1;	}
 
 	switch( grt )
@@ -81,21 +83,15 @@ int bmGifReadGif(		BitmapDescription *	bd,
 		bd->bdYResolution= 72;
 
 		bd->bdBitsPerSample= 8;
-		bd->bdSamplesPerPixel= 3;
-		bd->bdBitsPerPixel= 8;
+		if  ( bd->bdHasAlpha )
+		    { bd->bdBitsPerPixel= 16;	}
+		else{ bd->bdBitsPerPixel= 8;	}
 		bd->bdColorEncoding= BMcoRGB8PALETTE;
 		bd->bdColorCount= 256;
 		bd->bdRGB8Palette= (RGB8Color *)0;
-		bd->bdBytesPerRow= bd->bdPixelsWide;
-		bd->bdBufferLength= bd->bdPixelsHigh* bd->bdBytesPerRow;
 
-		if  ( bd->bdHasAlpha )
-		    {
-		    bd->bdSamplesPerPixel= 4;
-		    bd->bdBitsPerPixel *= 2;
-		    bd->bdBytesPerRow *= 2;
-		    bd->bdBufferLength *= 2;
-		    }
+		if  ( bmCalculateSizes( bd ) )
+		    { LDEB(1); return -1;	}
 
 		buffer= (unsigned char *)malloc( bd->bdBufferLength );
 		if  ( ! buffer )
@@ -439,7 +435,7 @@ int bmGifWriteGif(		const BitmapDescription *	bd,
     else{
 	unsigned char *		line;
 
-	line= (unsigned char *)malloc( bd->bdPixelsWide );
+	line= (unsigned char *)malloc( bd->bdPixelsWide+ 7 );
 	if  ( ! line )
 	    { XDEB(line); EGifCloseFile( gft ); return -1; }
 
