@@ -8,14 +8,14 @@
 
 #   include	<appDebugon.h>
 
-int bmSelect(	unsigned char **		pBuffer,
-		BitmapDescription *		bdOut,
-		const unsigned char *		bufIn,
+int bmSelect(	BitmapDescription *		bdOut,
+		unsigned char **		pBuffer,
 		const BitmapDescription *	bdIn,
+		const unsigned char *		bufIn,
 		int				x0,
 		int				y0,
 		int				wide,
-		int				high		)
+		int				high )
     {
     BitmapDescription	bd;
     int			bytesPerRow;
@@ -24,6 +24,14 @@ int bmSelect(	unsigned char **		pBuffer,
     unsigned char *	buffer;
     int			row;
 
+    if  ( x0+ wide > bdIn->bdPixelsWide	||
+	  y0+ high > bdIn->bdPixelsHigh )
+	{
+	LLLLDEB(x0,y0,wide,high);
+	LLDEB(bdIn->bdPixelsWide,bdIn->bdPixelsHigh);
+	return -1;
+	}
+
     /************************************************************/
     /*  Derive properties of output bitmap from input.		*/
     /************************************************************/
@@ -67,12 +75,15 @@ int bmSelect(	unsigned char **		pBuffer,
 	    to= buffer+ row* bytesPerRow;
 	    from= bufIn+ ( row+ y0 )* bdIn->bdBytesPerRow+ originByte;
 
+	    *to=   *from << originBit; from++;
+
 	    for ( byte= 1; byte < bytesPerRow; byte++ )
 		{
-		*to= *(from++) << originBit;
-		*(to++) |= *from >> rightShift;
+		*to |= *from >> rightShift; to++;
+		*to=   *from << originBit; from++;
 		}
-	    *to= *from << originBit;
+
+	    *to |= *from >> rightShift;
 	    }
 	}
 
