@@ -96,6 +96,24 @@ extern int appDebugSetFile(	const char *	filename,
 			    #c2, DEBC(c2),			\
 			    #c3, DEBC(c3)			))
 
+# define LCLDEB(l1,c2,l3) APP_DEB(appDebug( "%s(%3d)"		\
+			    DFML				\
+			    DFMC				\
+			    DFML				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBL(l1),			\
+			    #c2, DEBC(c2),			\
+			    #l3, DEBL(l3)			))
+
+# define LLCDEB(l1,l2,c3) APP_DEB(appDebug( "%s(%3d)"		\
+			    DFML				\
+			    DFML				\
+			    DFMC				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBL(l1),			\
+			    #l2, DEBL(l2),			\
+			    #c3, DEBC(c3)			))
+
 # define SSDEB(s1,s2) APP_DEB(appDebug( "%s(%3d)"		\
 			    DFMS				\
 			    DFMS				\
@@ -391,6 +409,17 @@ extern int appDebugSetFile(	const char *	filename,
 			    #l3, DEBL(l3),			\
 			    #l4, DEBL(l4)			))
 
+# define SLLXDEB(s1,l2,l3,l4) APP_DEB(appDebug( "%s(%3d)"	\
+			    DFMS				\
+			    DFML				\
+			    DFML				\
+			    DFMX				\
+			    "\n", __FILE__, __LINE__,		\
+			    #s1, DEBS(s1),			\
+			    #l2, DEBL(l2),			\
+			    #l3, DEBL(l3),			\
+			    #l4, DEBX(l4)			))
+
 # define LLLSDEB(l1,l2,l3,s4) APP_DEB(appDebug( "%s(%3d)"	\
 			    DFML				\
 			    DFML				\
