@@ -1,15 +1,10 @@
 #   include	"indConfig.h"
 
-#   include	<appDebugoff.h>
 #   include	"indlocal.h"
+#   include	<appDebugoff.h>
 
-#   ifdef	__STDC__					/*  {{	*/
-    static void	indTLDivide( IND *, int, int );
-    static void indTLClaim( IND *, int, int, int );
-#   else							/*  }{	*/
-    static void	indTLDivide();
-    static void indTLClaim();
-#   endif							/*  }}	*/
+static void	indTLDivide( IND *, int, int );
+static void indTLClaim( IND *, int, int, int );
 
 /************************************************************************/
 /*  The series of links is divided into groups.				*/
