@@ -28,6 +28,8 @@
 #include <string.h>
 #include "regmagic.h"
 
+#include <appDebugon.h>
+
 #ifndef CHARBITS
 #define	UCHARAT(p)	((int)*(unsigned char *)(p))
 #else
@@ -48,11 +50,11 @@ void regSubstitute(	regProg *	prog,
 	register int len;
 
 	if (prog == NULL || source == NULL || dest == NULL) {
-		regErrorMessage("NULL parm to regsub");
+		SDEB("NULL parm to regsub");
 		return;
 	}
 	if (UCHARAT(prog->rpProgram) != MAGIC) {
-		regErrorMessage("damaged regexp fed to regsub");
+		SDEB("damaged regexp fed to regsub");
 		return;
 	}
 
@@ -75,7 +77,7 @@ void regSubstitute(	regProg *	prog,
 			(void) strncpy(dst, (char *)prog->rpStartp[no], len);
 			dst += len;
 			if (len != 0 && *(dst-1) == '\0') {	/* strncpy hit NUL. */
-				regErrorMessage("damaged match string");
+				SDEB("damaged match string");
 				return;
 			}
 		}
